#! /usr/bin/env python
# coding=utf-8
import os
import socket
import threading
import json
import logging
import time

from   RobotControl.robotcontrol import *


# 创建一个全局log
logger = logging.getLogger()

# 创建线程锁
clients_lock = threading.Lock()


class ClientManager:
    def __init__(self):
        self._clients = []
        self.client_count = 0
        self.workers_maxnum = 5

	#添加客户端
    def add_client(self, connection, client_addr):
        with clients_lock:
            self._clients.append((connection, client_addr))
            self.client_count += 1
            logger.info("client:{0} is added into ClientManager".format(client_addr))
	
	#移除客户端
    def remove_client(self, connection, client_addr):
        with clients_lock:
            # Raises ValueError if the value is not present.
            if (connection, client_addr) in self._clients:
                self._clients.remove((connection, client_addr))
                self.client_count -= 1
                logger.info("client:{0} is removed from ClientManager".format(client_addr))
            else:
                logger.error("client: {0} is not contained in ClientManager".format(client_addr))

	#广播
    def broadcast(self, event):
        with clients_lock:
            for client in self._clients:
                client[0].sendall(RobotResponse(RobotJasonServerError.RESUTL_CALL_BACK,
                                                "robot callback event={0}".format(event['content'])).get_response())




# noinspection SpellCheckingInspection,PyUnresolvedReferences
class RobotJsonServer:
    def __init__(self, listen_port=3333, robot_ip='localhost', robot_port=8899):
        self.listen_port = listen_port
        self.robot_ip = robot_ip
        self.robot_port = robot_port
        self.robot = None
        self.listen_fd = None
        self.client_manager = ClientManager()
        self.robot_modbus_client = {}

    def __del__(self):
        del self.client_manager

    def stratup(self):
        # 系统初始化
        Auboi5Robot.initialize()

        # 创建机械臂实例
        self.robot = Auboi5Robot()

        # 创建上下文
        handle = self.robot.create_context()
        logger.info("robot RSHD={0}".format(handle))

        # 打印上下文
        logger.info("robot.rshd={0}".format(handle))

		#登录　　连接机械臂服务器
        result = self.robot.connect(self.robot_ip, self.robot_port)

        # 使能回调函数
        # self.robot.enable_robot_event()

        # 在robotjsonserver层重新设置回调函数,关闭在robotcontrol层的设置
        self.robot.set_robot_event_callback(self.event_callback)

        if result != RobotErrorType.RobotError_SUCC:
            logger.info(
                "RobotJsonServer connect robot server:{0}:{1} failed.".format(self.robot_ip[0], self.robot_port[0]))
        else:
            # 机械臂链接成功，启动socket的listen线程
            listen_thread = threading.Thread(target=self.listen_thread)
            listen_thread.setDaemon(True)
            listen_thread.start()
        return result

    def event_callback(self, event):
        # 更新最新事件
        logger.info("robot callback event = {0}".format(event))
        if event['type'] not in RobotEventType.NoError:
            self.robot.last_error = RobotError(event['type'], event['code'], event['content'])
        else:
            self.robot.last_event = RobotEvent(event['type'], event['code'], event['content'])
        # 向全部客户端广播事件
        self.client_manager.broadcast(event)

    def shutdown(self):
        # 断开服务器链接
        if self.robot.connected:
            # 关闭机械臂
            self.robot.robot_shutdown()
            # 断开机械臂链接
            self.robot.disconnect()
        # 释放资源
        Auboi5Robot.uninitialize()

        self.listen_fd.close()

    def listen_thread(self):
        try:
			#创建Socket
            self.listen_fd = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            logger.info("Bind Socket Port:{0}".format(self.listen_port))
            
            #绑定PORT
            self.listen_fd.bind(('0.0.0.0', self.listen_port))
            self.listen_fd.listen(5)
            logger.info("Robot Json Server Listening...")
            
            while True:
                connection, client_addr = self.listen_fd.accept()
                self.client_manager.add_client(connection, client_addr)
                logger.info("client:{0} connected.".format(client_addr))
                
                # 设置客户端最大数目
                if self.client_manager.client_count > self.client_manager.workers_maxnum:
                    logger.info("The number of clients must be less than {0}".format(self.client_manager.workers_maxnum))
                    self.client_manager.remove_client(connection, client_addr)
                    connection.sendall(RobotResponse(RobotJasonServerError.RESUTL_ERROR_CLIENTS_NUMBER,
                                                     "The number of client must be less than {0}".format(
                                                         self.client_manager.workers_maxnum)).get_response())
                    connection.close()
                    continue
                    
                # 启动客户端线程
                client_worker = RobotClientWorker(self.robot, self.client_manager, self.robot_modbus_client, connection, client_addr)
                client_worker.setDaemon(True)
                client_worker.start()
        except socket.error, error_msg:
            # 打印socket出错信息
            logger.error("Listen Thread Error. Error Code : {0}, Message: {1}".format(error_msg[0], error_msg[1]))
            # 断开机械臂服务器链接
            self.shutdown()
            # 监听端口失败,退出系统
            # noinspection PyProtectedMember
            os._exit(1)




class RobotJasonServerError:
    # 成功
    RESUTL_SUCC = 0
    # 错误信息起始值
    RESUTL_BASE = 3000
    # 错误的指令字符串
    RESUTL_ERROR_BAD_COMMAND_STRING = RESUTL_BASE + 1
    # 未知指令
    RESUTL_ERROR_UNKNOWN_COMMAND = RESUTL_BASE + 2
    # 参数错误
    RESUTL_ERROR_ARGS = RESUTL_BASE + 2
    # 运动过程错误
    RESUTL_ERROR_MOVE = RESUTL_BASE + 3
    # 回调事件
    RESUTL_CALL_BACK = RESUTL_BASE + 4
    # 客户端个数超限
    RESUTL_ERROR_CLIENTS_NUMBER = RESUTL_BASE + 5
    # 尚未连接modbus服务端
    RESUTL_ERROR_DISCONNECT_TO_MODBUS_SERVER = RESUTL_BASE + 6
    # 连接modbus服务端错误
    RESUTL_ERROR_CONNECT_TO_MODBUS_SERVER = RESUTL_BASE + 7

    def __init__(self):
        pass




class RobotResponse:
    succ = 'succ'
    failed = 'failed'

    def __init__(self, return_code, return_msg):
        self.return_code = return_code
        self.return_msg = return_msg

    def get_response(self):
        return json.dumps({'ret': self.return_code, 'msg': self.return_msg})

    def __str__(self):
        return self.get_response()


class RobotStatusResponse:
    RobotStatusJson = ("Stopped", "Running", "Pause", "Resumed")

    def __init__(self, return_state):
        self.return_state = return_state

    def get_response(self):
        return self.RobotStatusJson[self.return_state]


class RobotRunningModeResponse:
    RobotRunningModeJson = ("RobotModeSimulator", "RobotModeReal")

    def __init__(self, return_running_mode):
        self.return_running_mode = return_running_mode

    def get_response(self):
        return self.RobotRunningModeJson[self.return_running_mode]


# 机械臂控制指令字符串
_robot_command = (
    'robot_startup', 'robot_shutdown', 'init_profile', 'set_joint_maxacc', 'get_joint_maxacc', 'set_joint_maxvelc',
    'get_joint_maxvelc', 'set_end_max_line_acc', 'get_end_max_line_acc', 'set_end_max_line_velc',
    'get_end_max_line_velc', 'set_end_max_angle_acc', 'get_end_max_angle_acc', 'set_end_max_angle_velc',
    'get_end_max_angle_velc', 'move_joint', 'move_line', 'move_rotate', 'remove_all_waypoint', 'add_waypoint',
    'set_blend_radius', 'set_circular_loop_times', 'set_user_coord', 'set_base_coord', 'check_user_coord',
    'set_relative_offset_on_user', 'set_relative_offset_on_base', 'move_track', 'move_stop', 'move_pause',
    'move_continue', 'collision_recover', 'forward_kin', 'inverse_kin', 'base_to_user', 'user_to_base',
    'base_to_base_additional_tool', 'rpy_to_quaternion', 'quaternion_to_rpy', 'set_none_tool_kinematics_param',
    'set_tool_kinematics_param', 'get_tool_kinematics_param', 'get_robot_state', 'set_none_tool_dynamics_param',
    'set_tool_dynamics_param', 'get_tool_dynamics_param', 'get_work_mode', 'set_work_mode', 'set_collision_class',
    'is_have_real_robot', 'is_online_master_mode', 'get_joint_status', 'get_current_waypoint', 'get_board_io_config',
    'get_board_io_status', 'set_board_io_status', 'set_tool_power_type', 'get_tool_power_type', 'set_tool_io_type',
    'get_tool_power_voltage', 'get_tool_io_status', 'set_tool_io_status', 'clear_offline_track',
    'append_offline_track_waypoint', 'append_offline_track_file', 'startup_offline_track', 'stop_offline_track',
    'enter_tcp2canbus_mode', 'leave_tcp2canbus_mode', 'set_waypoint_to_canbus', 'set_no_arrival_ahead',
    'set_arrival_ahead_distance', 'set_arrival_ahead_time', 'connect_to_modbus_server', 'modbus_client_read_coils',
    'modbus_client_read_discrete_inputs', 'modbus_client_read_holding_registers', 'modbus_client_read_input_registers',
    'modbus_client_write_coil', 'modbus_client_write_register', 'modbus_client_write_coils', 'modbus_client_write_registers',
    'disconnect_to_modbus_server'
)


class RobotClientWorker(threading.Thread):
    def __init__(self, robot, client_manager, robot_modbus_client, connection, client_addr):
        threading.Thread.__init__(self)
        # 机械臂实例
        self.robot = robot
        # 客户端注册表
        self.client_manager = client_manager
        # modbus客户端
        self.robot_modbus_client = robot_modbus_client
        # 服务器地址
        self.connection = connection
        # 服务器端口
        self.client_addr = client_addr
        # 控制指令函数映射字典
        self._robot_handler = {
            'robot_startup': self._robot_startup,
            'robot_shutdown': self._robot_shutdown,
            'init_profile': self._init_profile,
            'set_joint_maxacc': self._set_joint_maxacc,
            'get_joint_maxacc': self._get_joint_maxacc,
            'set_joint_maxvelc': self._set_joint_maxvelc,
            'get_joint_maxvelc': self._get_joint_maxvelc,
            'set_end_max_line_acc': self._set_end_max_line_acc,
            'get_end_max_line_acc': self._get_end_max_line_acc,
            'set_end_max_line_velc': self._set_end_max_line_velc,
            'get_end_max_line_velc': self._get_end_max_line_velc,
            'set_end_max_angle_acc': self._set_end_max_angle_acc,
            'get_end_max_angle_acc': self._get_end_max_angle_acc,
            'set_end_max_angle_velc': self._set_end_max_angle_velc,
            'get_end_max_angle_velc': self._get_end_max_angle_velc,
            'move_joint': self._move_joint,
            'move_line': self._move_line,
            'move_rotate': self._move_rotate,
            'remove_all_waypoint': self._remove_all_waypoint,
            'add_waypoint': self._add_waypoint,
            'set_blend_radius': self._set_blend_radius,
            'set_circular_loop_times': self._set_circular_loop_times,
            'set_user_coord': self._set_user_coord,
            'set_base_coord': self._set_base_coord,
            'check_user_coord': self._check_user_coord,
            'set_relative_offset_on_user': self._set_relative_offset_on_user,
            'set_relative_offset_on_base': self._set_relative_offset_on_base,
            'move_track': self._move_track,
            'move_stop': self._move_stop,
            'move_pause': self._move_pause,
            'move_continue': self._move_continue,
            'collision_recover': self._collision_recover,
            'forward_kin': self._forward_kin,
            'inverse_kin': self._inverse_kin,
            'base_to_user': self._base_to_user,
            'user_to_base': self._user_to_base,
            'base_to_base_additional_tool': self._base_to_base_additional_tool,
            'rpy_to_quaternion': self._rpy_to_quaternion,
            'quaternion_to_rpy': self._quaternion_to_rpy,
            'set_none_tool_kinematics_param': self._set_none_tool_kinematics_param,
            'set_tool_kinematics_param': self._set_tool_kinematics_param,
            'get_tool_kinematics_param': self._get_tool_kinematics_param,
            'set_none_tool_dynamics_param': self._set_none_tool_dynamics_param,
            'set_tool_dynamics_param': self._set_tool_dynamics_param,
            'get_tool_dynamics_param': self._get_tool_dynamics_param,
            'get_robot_state': self._get_robot_state,
            'get_work_mode': self._get_work_mode,
            'set_work_mode': self._set_work_mode,
            'set_collision_class': self._set_collision_class,
            'is_have_real_robot': self._is_have_real_robot,
            'is_online_master_mode': self._is_online_master_mode,
            'get_joint_status': self._get_joint_status,
            'get_current_waypoint': self._get_current_waypoint,
            'get_board_io_config': self._get_board_io_config,
            'get_board_io_status': self._get_board_io_status,
            'set_board_io_status': self._set_board_io_status,
            'set_tool_power_type': self._set_tool_power_type,
            'get_tool_power_type': self._get_tool_power_type,
            'set_tool_io_type': self._set_tool_io_type,
            'get_tool_power_voltage': self._get_tool_power_voltage,
            'get_tool_io_status': self._get_tool_io_status,
            'set_tool_io_status': self._set_tool_io_status,
            'clear_offline_track': self._clear_offline_track,
            'append_offline_track_waypoint': self._append_offline_track_waypoint,
            'append_offline_track_file': self._append_offline_track_file,
            'startup_offline_track': self._startup_offline_track,
            'stop_offline_track': self._stop_offline_track,
            'enter_tcp2canbus_mode': self._enter_tcp2canbus_mode,
            'leave_tcp2canbus_mode': self._leave_tcp2canbus_mode,
            'set_waypoint_to_canbus': self._set_waypoint_to_canbus,
            'set_no_arrival_ahead': self._set_no_arrival_ahead,
            'set_arrival_ahead_distance': self._set_arrival_ahead_distance,
            'set_arrival_ahead_time': self._set_arrival_ahead_time,
            'connect_to_modbus_server': self._connect_to_modbus_server,
            'modbus_client_read_coils': self._modbus_client_read_coils,
            'modbus_client_read_discrete_inputs': self._modbus_client_read_discrete_inputs,
            'modbus_client_read_holding_registers': self._modbus_client_read_holding_registers,
            'modbus_client_read_input_registers': self._modbus_client_read_input_registers,
            'modbus_client_write_coil': self._modbus_client_write_coil,
            'modbus_client_write_register': self._modbus_client_write_register,
            'modbus_client_write_coils': self._modbus_client_write_coils,
            'modbus_client_write_registers': self._modbus_client_write_registers,
            'disconnect_to_modbus_server': self._disconnect_to_modbus_server
        }

        # 运动轨迹类型映射字典
        self._tracktype_handler = {"ARC_CIR": RobotMoveTrackType.ARC_CIR,
                                   "CARTESIAN_MOVEP": RobotMoveTrackType.CARTESIAN_MOVEP
                                   }

    def run(self):
        self.client_handler()

    def send_response(self, code, msg):
        with clients_lock:
            self.connection.sendall(RobotResponse(code, msg).get_response())

	#业务处理
    def client_handler(self):
        try:
            while True:
                data_buf = self.connection.recv(4096)
                if not data_buf:
                    logger.info("clent:{0} disconnected".format(self.client_addr))
                    break
                else:
                    self.process_json_cmd(data_buf)
        except socket.error, error_msg:
            # noinspection PyUnresolvedReferences,PyUnresolvedReferences
            logger.error("robot client thread except. Error Code : {0} Message: {1}".format(
                error_msg[0], error_msg[1]))
            logger.info("connection.close")
        finally:
            self.client_manager.remove_client(self.connection, self.client_addr)
            self.connection.close()
            logger.info("robot client thread exit. client:{0}".format(self.client_addr))

    def process_json_cmd(self, data_buf='{}'):
        try:
            # json command string to python object
            robot_cmd = json.loads(data_buf)
            if 'command' in robot_cmd.keys() and robot_cmd['command'] in _robot_command:
                self._robot_handler[robot_cmd['command']](robot_cmd)
            else:
                self.send_response(RobotJasonServerError.RESUTL_ERROR_UNKNOWN_COMMAND, 'Unknown Command')

        except TypeError, msg:
            logger.error("parameter tpye error:" + msg[0])
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, 'Parameter Tpye Error:' + msg[0])

        except RobotError, e:
            logger.error("{0} robot Event:{1}".format(self.robot.get_local_time(), e))
            self.send_response(RobotJasonServerError.RESUTL_ERROR_MOVE, 'Move Error!')

        except ValueError, msg:
            logger.error("parse json error:" + msg[0])
            logger.error("json data:" + data_buf)
            self.send_response(RobotJasonServerError.RESUTL_ERROR_BAD_COMMAND_STRING, 'Bad Json String!')

        except Exception, msg:
            logger.error("Unexpected Error:" + msg[0])
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, "Unexpected Error:" + msg[0])

    @staticmethod
    def list_to_tuple(robot_cmd={}):
        if 'user_coord' in robot_cmd.keys():
            robot_cmd['user_coord']['calibrate_points']['point1'] = tuple(
                robot_cmd['user_coord']['calibrate_points']['point1'])
            robot_cmd['user_coord']['calibrate_points']['point2'] = tuple(
                robot_cmd['user_coord']['calibrate_points']['point2'])
            robot_cmd['user_coord']['calibrate_points']['point3'] = tuple(
                robot_cmd['user_coord']['calibrate_points']['point3'])
            robot_cmd['user_coord']['tool_desc']['pos'] = tuple(robot_cmd['user_coord']['tool_desc']['pos'])
            robot_cmd['user_coord']['tool_desc']['ori'] = tuple(robot_cmd['user_coord']['tool_desc']['ori'])
        if 'pos' in robot_cmd.keys():
            robot_cmd['pos'] = tuple(robot_cmd['pos'])
        if 'ori' in robot_cmd.keys():
            robot_cmd['ori'] = tuple(robot_cmd['ori'])
        if 'user_tool' in robot_cmd.keys():
            robot_cmd['user_tool']['pos'] = tuple(robot_cmd['user_tool']['pos'])
            robot_cmd['user_tool']['ori'] = tuple(robot_cmd['user_tool']['ori'])
        if 'rpy' in robot_cmd.keys():
            robot_cmd['rpy'] = tuple(robot_cmd['rpy'])
        if 'tool_end_param' in robot_cmd.keys():
            robot_cmd['tool_end_param']['pos'] = tuple(robot_cmd['tool_end_param']['pos'])
            robot_cmd['tool_end_param']['ori'] = tuple(robot_cmd['tool_end_param']['ori'])
        if 'tool_dynamics' in robot_cmd.keys():
            robot_cmd['tool_dynamics']['position'] = tuple(robot_cmd['tool_dynamics']['position'])
            robot_cmd['tool_dynamics']['inertia'] = tuple(robot_cmd['tool_dynamics']['inertia'])
        if 'waypoints' in robot_cmd.keys():
            for i in range(len(robot_cmd['waypoints'])):
                robot_cmd['waypoints'][i] = tuple(robot_cmd['waypoints'][i])
            robot_cmd['waypoints'] = tuple(robot_cmd['waypoints'])



    def _robot_startup(self, robot_cmd):
        """
        * FUNCTION:    _robot_startup
        * DESCRIPTION: 启动机械臂
        * INPUTS:
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       默认：
        *              tool_dynamics = 位置，单位(m) ：{"position": (0.0, 0.0, 0.0),
        *                              负载，单位(kg)： "payload": 1.0,
        *                              惯量，          "inertia": (0.0, 0.0, 0.0, 0.0, 0.0, 0.0)}
        * EXAMPLE:      {"command":"robot_startup"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.robot_startup()
        if result == RobotErrorType.RobotError_SUCC:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)

    def _robot_shutdown(self, robot_cmd):
        """
        * FUNCTION:    _robot_shutdown
        * DESCRIPTION: 关闭机械臂
        * INPUTS:
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * EXAMPLE:      {"command":"robot_shutdown"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.robot_shutdown()
        if result == RobotErrorType.RobotError_SUCC:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)

    def _init_profile(self, robot_cmd):
        """"
        * FUNCTION:    _init_profile
        * DESCRIPTION: 初始化机械臂控制全局属性
        * INPUTS:
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       调用成功后，系统会自动清理掉之前设置的用户坐标系，
        *              速度，加速度等等属性
        * EXAMPLE:      {"command":"init_profile"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.init_profile()
        if result == RobotErrorType.RobotError_SUCC:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)

    def _set_joint_maxacc(self, robot_cmd):
        """
        * FUNCTION:    _set_joint_maxacc
        * DESCRIPTION: 设置六个关节的最大加速度
        * INPUTS:      robot_cmd:机械臂控制指令
                       robot_cmd = {
                            "command":"set_joint_maxacc",
                            "joint_maxacc":[0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000]
                        }
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       joint_maxacc:六个关节的最大加速度，单位(rad/s)
        * EXAMPLE:     {"command":"set_joint_maxacc", "joint_maxacc":[1.0, 1.0, 1.0, 1.0, 1.0, 1.0]}
        """
        if 'joint_maxacc' in robot_cmd.keys():
            logger.info('set joint maxacc : {0}'.format(robot_cmd['joint_maxacc']))
            result = self.robot.set_joint_maxacc(robot_cmd['joint_maxacc'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set joint maxacc error :{0}".format(robot_cmd['joint_maxacc']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_joint_maxacc bad args, args={0}".format(robot_cmd))

    def _get_joint_maxacc(self, robot_cmd):
        """
        * FUNCTION:    _get_joint_maxacc
        * DESCRIPTION: 获取六个关节的最大加速度
        * INPUTS:
        * RETURNS:     成功返回: 六个关节的最大加速度单位(rad/s^2)
        *              失败返回: None
        * EXAMPLE:     {"command":"get_joint_maxacc"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.get_joint_maxacc()
        if result is not None:
            self.send_response(result, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("get joint maxacc error")

    def _set_joint_maxvelc(self, robot_cmd):
        """
        * FUNCTION:    _set_joint_maxvelc
        * DESCRIPTION: 设置六个关节的最大速度
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       joint_maxvelc:六个关节的最大速度，单位(rad/s)
        * EXAMPLE:     {"command":"set_joint_maxvelc", "joint_maxvelc":[1.0, 1.0, 1.0, 1.0, 1.0, 1.0]}
        """
        if 'joint_maxvelc' in robot_cmd.keys():
            logger.info('set joint maxvelc : {0}'.format(robot_cmd['joint_maxvelc']))
            result = self.robot.set_joint_maxvelc(robot_cmd['joint_maxvelc'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotErrorType.RobotError_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set joint maxvelc error :{0}".format(robot_cmd['joint_maxvelc']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_joint_maxvelc bad args, args={0}".format(robot_cmd))

    def _get_joint_maxvelc(self, robot_cmd):
        """
        * FUNCTION:    _get_joint_maxvelc
        * DESCRIPTION: 获取六个关节的最大速度
        * INPUTS:
        * RETURNS:     成功返回: 六个关节的最大速度单位(rad/s)
        *              失败返回: None
        * EXAMPLE:     {"command":"get_joint_maxvelc"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.get_joint_maxvelc()
        if result is not None:
            self.send_response(result, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("get joint maxvelc error")

    def _set_end_max_line_acc(self, robot_cmd):
        """
        * FUNCTION:    _set_end_max_line_acc
        * DESCRIPTION: 设置机械臂末端最大线加速度
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       end_maxacc:末端最大加线速度，单位(m/s^2)
        * EXAMPLE:     {"command":"set_end_max_line_acc", "end_maxacc":0.2}
        """
        if 'end_maxacc' in robot_cmd.keys():
            logger.info('set end max line acc : {0}'.format(robot_cmd['end_maxacc']))
            result = self.robot.set_end_max_line_acc(robot_cmd['end_maxacc'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set end max line acc error: {0}".format(robot_cmd['end_maxacc']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_end_max_line_acc bad args, args={0}".format(robot_cmd))

    def _get_end_max_line_acc(self, robot_cmd):
        """
        * FUNCTION:    _get_end_max_line_acc
        * DESCRIPTION: 获取机械臂末端最大线加速度
        * INPUTS:
        * RETURNS:     成功返回: 机械臂末端最大加速度，单位(m/s^2)
        *              失败返回: None
        * EXAMPLE:     {"command":"get_end_max_line_acc"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.get_end_max_line_acc()
        if result is not None:
            self.send_response(result, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("get end max line acc error")

    def _set_end_max_line_velc(self, robot_cmd):
        """
        * FUNCTION:    _set_end_max_line_velc
        * DESCRIPTION: 设置机械臂末端最大线速度
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: (RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       end_maxvelc:末端最大加线速度，单位(m/s)
        * EXAMPLE:     {"command":"set_end_max_line_velc", "end_maxvelc":0.2}
        """
        if 'end_maxvelc' in robot_cmd.keys():
            logger.info('set end max line velc : {0}'.format(robot_cmd['end_maxvelc']))
            result = self.robot.set_end_max_line_velc(robot_cmd['end_maxvelc'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set end max line velc error: {0}".format(robot_cmd['end_maxvelc']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_end_max_line_velc bad args, args={0}".format(robot_cmd))

    def _get_end_max_line_velc(self, robot_cmd):
        """
        * FUNCTION:    _get_end_max_line_velc
        * DESCRIPTION: 获取机械臂末端最大线速度
        * INPUTS:
        * RETURNS:     成功返回: 机械臂末端最大加速度，单位(m/s)
        *              失败返回: None
        * EXAMPLE:     {"command":"get_end_max_line_velc"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.get_end_max_line_velc()
        if result is not None:
            self.send_response(result, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("get end max line velc error")

    def _set_end_max_angle_acc(self, robot_cmd):
        """
        * FUNCTION:    _set_end_max_angle_acc
        * DESCRIPTION: 设置机械臂末端最大角加速度
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       end_maxacc:末端最大角加速度，单位(rad/s^2)
        * EXAMPLE:     {"command":"set_end_max_angle_acc", "end_maxacc":0.2}
        """
        if 'end_maxacc' in robot_cmd.keys():
            logger.info('set end max angle acc : {0}'.format(robot_cmd['end_maxacc']))
            result = self.robot.set_end_max_angle_acc(robot_cmd['end_maxacc'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set end max angle acc error: {0}".format(robot_cmd['end_maxacc']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_end_max_angle_acc bad args, args={0}".format(robot_cmd))

    def _get_end_max_angle_acc(self, robot_cmd):
        """
        * FUNCTION:    _get_end_max_angle_acc
        * DESCRIPTION: 获取机械臂末端最大角加速度
        * INPUTS:
        * RETURNS:     成功返回: 机械臂末端最大角加速度，单位(rad/s^2)
        *              失败返回: None
        * EXAMPLE:     {"command":"get_end_max_angle_acc"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.get_end_max_angle_acc()
        if result is not None:
            self.send_response(result, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("get end max angle acc error")

    def _set_end_max_angle_velc(self, robot_cmd):
        """
        * FUNCTION:    _set_end_max_angle_velc
        * DESCRIPTION: 设置机械臂末端最大角速度
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       end_maxvelc:末端最大加线速度，单位(rad/s)
        * EXAMPLE:     {"command":"set_end_max_angle_velc", "end_maxvelc":0.2}
        """
        if 'end_maxvelc' in robot_cmd.keys():
            logger.info('set end max angle velc : {0}'.format(robot_cmd['end_maxvelc']))
            result = self.robot.set_end_max_angle_velc(robot_cmd['end_maxvelc'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set end max angle velc error: {0}".format(robot_cmd['end_maxvelc']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_end_max_angle_velc bad args, args={0}".format(robot_cmd))

    def _get_end_max_angle_velc(self, robot_cmd):
        """
        * FUNCTION:    _get_end_max_angle_velc
        * DESCRIPTION: 获取机械臂末端最大角速度
        * INPUTS:
        * RETURNS:     成功返回: 机械臂末端最大角速度，单位(rad/s)
        *              失败返回: None
        * EXAMPLE:     {"command":"get_end_max_angle_velc"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.get_end_max_angle_velc()
        if result is not None:
            self.send_response(result, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("get end max angle velc error")

    def _move_joint(self, robot_cmd):
        """
         * FUNCTION:    _move_joint
         * DESCRIPTION: 机械臂轴动
         * INPUTS:      robot_cmd:机械臂控制指令
         * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
         *              失败返回: 其他
         * NOTES:       move_joint:机械臂轴动
                        joint_radian:六个关节的关节角，单位(rad)
         * EXAMPLE:     {"command":"move_joint", "joint_radian":[0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000]}
         """
        if 'joint_radian' in robot_cmd.keys():
            logger.info("move joint to {0}".format(robot_cmd['joint_radian']))
            result = self.robot.move_joint(robot_cmd['joint_radian'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("move joint to {0} error".format(robot_cmd['joint_radian']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("move_joint bad args, args={0}".format(robot_cmd))

    def _move_line(self, robot_cmd):
        """
         * FUNCTION:    _move_line
         * DESCRIPTION: 机械臂保持当前姿态直线运动
         * INPUTS:      robot_cmd:机械臂控制指令
         * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
         *              失败返回: 其他
         * NOTES:       move_line:机械臂保持当前姿态直线运动
                        joint_radian:六个关节的关节角，单位(rad)
         * EXAMPLE:     {"command":"move_line", "joint_radian":[0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000]}
         """
        if 'joint_radian' in robot_cmd.keys():
            logger.info("move line to {0}".format(robot_cmd['joint_radian']))
            result = self.robot.move_line(robot_cmd['joint_radian'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("move line to {0} error".format(robot_cmd['joint_radian']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("move_line bad args, args={0}".format(robot_cmd))

    def _move_rotate(self, robot_cmd):
        """
         * FUNCTION:    _move_rotate
         * DESCRIPTION: 保持当前位置变换姿态做旋转运动
         * INPUTS:      robot_cmd:机械臂控制指令
         * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
         *              失败返回: 其他
         * NOTES:       user_coord:用户坐标系
                        rotate_axis:转轴(x,y,z) 例如：(1,0,0)表示沿Y轴转动
                        rotate_angle:旋转角度 单位（rad）
         * EXAMPLE:     {"command": "move_rotate",
                         "user_coord" : {"coord_type":       2,
                                         "calibrate_method": 0,
                                         "calibrate_points": {"point1": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                                                              "point2": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                                                              "point3": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]},
                                          "tool_desc":       {"pos": [0.0, 0.0, 0.0],
                                                              "ori": [1.0, 0.0, 0.0, 0.0]}
                                          },
                         "rotate_axis":[1,0,0],
                         "rotate_angle":0.5
                         }
        """
        if 'user_coord' and 'rotate_axis' and 'rotate_angle' in robot_cmd.keys():
            self.list_to_tuple(robot_cmd)
            logger.info("move rotate :user_coord {0} rotate_axis {1}, rotate_angle {2}".format(
                robot_cmd['user_coord'], robot_cmd['rotate_axis'], robot_cmd['rotate_angle']))
            result = self.robot.move_rotate(robot_cmd['user_coord'], robot_cmd['rotate_axis'],
                                            robot_cmd['rotate_angle'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("move rotate :user_coord {0} rotate_axis {1}, rotate_angle {2}".format(
                    robot_cmd['user_coord'], robot_cmd['rotate_axis'], robot_cmd['rotate_angle']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("move_rotate bad args, args={0}".format(robot_cmd))

    def _clear_offline_track(self, robot_cmd):
        """"
        * FUNCTION:    _clear_offline_track
        * DESCRIPTION: 清理服务器上的非在线轨迹运动数据
        * INPUTS:
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * EXAMPLE:      {"command":"clear_offline_track"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.clear_offline_track()
        if result == RobotErrorType.RobotError_SUCC:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)

    def _append_offline_track_waypoint(self, robot_cmd):
        """
        * FUNCTION:    _append_offline_track_waypoint
        * DESCRIPTION: 向服务器添加非在线轨迹运动路点
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       waypoints 非在线轨迹运动路点元祖(可包含小于3000个路点), 单位:弧度
        *              例如:((0.0, 0.0, 0.0, 0.0, 0.0, 0.0),
        *                    (0.0,-0.000001,-0.000001,0.000001,-0.000001, 0.0))
        * EXAMPLE:     {"command":"append_offline_track_waypoint", "waypoints":[[0.0, 0.0, 0.0, 0.0, 0.0, 0.0],[0.0,-0.000001,-0.000001,0.000001,-0.000001, 0.0]]}
        """
        if 'waypoints' in robot_cmd.keys():
            self.list_to_tuple(robot_cmd)
            logger.info("append offline track waypoint {0}".format(robot_cmd['waypoints']))
            result = self.robot.append_offline_track_waypoint(robot_cmd['waypoints'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("append_offline_track_waypoint {0} error".format(robot_cmd['waypoints']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("append offline track waypoint bad args, args={0}".format(robot_cmd))

    def _append_offline_track_file(self, robot_cmd):
        """
        * FUNCTION:    _append_offline_track_file
        * DESCRIPTION: 向服务器添加非在线轨迹运动路点文件
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       路点文件全路径,路点文件的每一行包含六个关节的关节角(弧度),用逗号隔开
        * EXAMPLE:     {"command":"append_offline_track_file", "track_file":"target-ok.txt"}
        """
        if 'track_file' in robot_cmd.keys():
            logger.info("append offline track file {0}".format(robot_cmd['track_file']))
            result = self.robot.append_offline_track_file(robot_cmd['track_file'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("append offline track file {0} error".format(robot_cmd['track_file']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("append_offline_track_file bad args, args={0}".format(robot_cmd))

    def _startup_offline_track(self, robot_cmd):
        """"
        * FUNCTION:    _startup_offline_track
        * DESCRIPTION: 通知服务器启动非在线轨迹运动
        * INPUTS:
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * EXAMPLE:      {"command":"startup_offline_track"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.startup_offline_track()
        if result == RobotErrorType.RobotError_SUCC:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)

    def _stop_offline_track(self, robot_cmd):
        """"
        * FUNCTION:    _stop_offline_track
        * DESCRIPTION: 通知服务器停止非在线轨迹运动
        * INPUTS:
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * EXAMPLE:      {"command":"stop_offline_track"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.stop_offline_track()
        if result == RobotErrorType.RobotError_SUCC:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)

    def _enter_tcp2canbus_mode(self, robot_cmd):
        """"
        * FUNCTION:    _enter_tcp2canbus_mode
        * DESCRIPTION: 通知服务器进入TCP2CANBUS透传模式
        * INPUTS:
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * EXAMPLE:      {"command":"enter_tcp2canbus_mode"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.enter_tcp2canbus_mode()
        if result == RobotErrorType.RobotError_SUCC:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)

    def _leave_tcp2canbus_mode(self, robot_cmd):
        """"
        * FUNCTION:    _leave_tcp2canbus_mode
        * DESCRIPTION: 通知服务器退出TCP2CANBUS透传模式
        * INPUTS:
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * EXAMPLE:      {"command":"leave_tcp2canbus_mode"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.leave_tcp2canbus_mode()
        if result == RobotErrorType.RobotError_SUCC:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)

    def _set_waypoint_to_canbus(self, robot_cmd):
        """
        * FUNCTION:    _set_waypoint_to_canbus
        * DESCRIPTION: 透传运动路点到CANBUS
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       joint_radian:六个关节的关节角，单位(rad)
        * EXAMPLE:     {"command":"set_waypoint_to_canbus", "joint_radian":[-0.000003, -0.127267, -1.321122, 0.376934, -1.570796, -0.000008]}
        """
        if 'joint_radian' in robot_cmd.keys():
            logger.info("set waypoint to canbus {0}".format(robot_cmd['joint_radian']))
            result = self.robot.set_waypoint_to_canbus(robot_cmd['joint_radian'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set waypoint to canbus {0} error".format(robot_cmd['joint_radian']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_waypoint_to_canbus bad args, args={0}".format(robot_cmd))

    def _get_current_waypoint(self, robot_cmd):
        """
        * FUNCTION:     _get_current_waypoint
        * DESCRIPTION:  获取机械臂当前状态信息
        * INPUTS:
        * RETURNS:     成功返回: 关节位置信息，结果为详见NOTES
        *              失败返回: None
        * NOTES:       六个关节角 {'joint': [1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
        *              位置 'pos': [-0.06403157614989634, -0.4185973810159096, 0.816883228463401],
        *              姿态 'ori': [-0.11863209307193756, 0.3820514380931854, 0.0, 0.9164950251579285]}
        * EXAMPLE:     {"command":"get_current_waypoint"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.get_current_waypoint()
        if result is not None:
            self.send_response(result, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("get current waypoint")

    def _remove_all_waypoint(self, robot_cmd):
        """
        * FUNCTION:    _remove_all_waypoint
        * DESCRIPTION: 清除所有已经设置的全局路点
        * INPUTS:
        * OUTPUTS:
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * EXAMPLE:     {"command":"remove_all_waypoint"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.remove_all_waypoint()
        if result == RobotErrorType.RobotError_SUCC:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)

    def _add_waypoint(self, robot_cmd):
        """
        * FUNCTION:    _add_waypoint
        * DESCRIPTION: 添加全局路点用于轨迹运动
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       joint_radian:六个关节的关节角，单位(rad)
        * EXAMPLE:     {"command":"add_waypoint", "joint_radian":[-0.000003, -0.127267, -1.321122, 0.376934, -1.570796, -0.000008]}
        """
        if 'joint_radian' in robot_cmd.keys():
            logger.info("add waypoint {0}".format(robot_cmd['joint_radian']))
            result = self.robot.add_waypoint(robot_cmd['joint_radian'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("add waypoint {0} error".format(robot_cmd['joint_radian']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("add_waypoint bad args, args={0}".format(robot_cmd))

    def _set_blend_radius(self, robot_cmd):
        """
        * FUNCTION:    _set_blend_radius
        * DESCRIPTION: 设置交融半径
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       blend_radius:交融半径，单位(m)
        * EXAMPLE:     {"command":"set_blend_radius", "blend_radius":0.01}
        """
        if 'blend_radius' in robot_cmd.keys():
            logger.info('set blend radius : {0}'.format(robot_cmd['blend_radius']))
            result = self.robot.set_blend_radius(robot_cmd['blend_radius'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set blend radius error :{0}".format(robot_cmd['blend_radius']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_blend_radius bad args, args={0}".format(robot_cmd))

    def _set_circular_loop_times(self, robot_cmd):
        """
        * FUNCTION:    _set_circular_loop_times
        * DESCRIPTION: 设置圆运动圈数
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        *
        * NOTES:       circular_count:圆的运动圈数
                       当circular_count大于0时，机械臂进行圆运动circular_count次
        *              当circular_count等于0时，机械臂进行圆弧轨迹运动
        * EXAMPLE:     {"command":"set_circular_loop_times", "circular_count":1}
        """
        if 'circular_count' in robot_cmd.keys():
            logger.info('set circular loop times : {0}'.format(robot_cmd['circular_count']))
            result = self.robot.set_circular_loop_times(robot_cmd['circular_count'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set circular loop times error :{0}".format(robot_cmd['circular_count']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_circular_loop_times bad args, args={0}".format(robot_cmd))

    def _set_user_coord(self, robot_cmd):
        """
        * FUNCTION:    _set_user_coord
        * DESCRIPTION: 设置用户坐标系
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       user_coord:用户坐标系
        * EXAMPLE:      {"command":    "set_user_coord",
                         "user_coord" : {"coord_type":       2,
                                         "calibrate_method": 0,
                                         "calibrate_points": {"point1": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                                                              "point2": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                                                              "point3": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]},
                                          "tool_desc":       {"pos": [0.0, 0.0, 0.0],
                                                              "ori": [1.0, 0.0, 0.0, 0.0]}
                                          }
                         }
        """
        if 'user_coord' in robot_cmd.keys():
            self.list_to_tuple(robot_cmd)
            logger.info("set user coord : {0}".format(robot_cmd['user_coord']))
            result = self.robot.set_user_coord(robot_cmd['user_coord'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set user coord : {0}".format(robot_cmd['user_coord']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_user_coord bad args, args={0}".format(robot_cmd))

    def _set_base_coord(self, robot_cmd):
        """
        * FUNCTION:    _set_base_coord
        * DESCRIPTION: 设置基座坐标系
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * EXAMPLE:      {"command": "set_base_coord"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.set_base_coord()
        if result == RobotErrorType.RobotError_SUCC:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("set user coord ")

    def _check_user_coord(self, robot_cmd):
        """
        * FUNCTION:    _check_user_coord
        * DESCRIPTION: 检查用户坐标系参数设置是否合理
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     合理返回: RobotJasonServerError.RESUTL_SUCC
        *              不合理返回: 其他
        * EXAMPLE:      {"command":    "check_user_coord",
                         "user_coord" : {"coord_type":       2,
                                         "calibrate_method": 0,
                                         "calibrate_points": {"point1": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                                                              "point2": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                                                              "point3": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]},
                                          "tool_desc":       {"pos": [0.0, 0.0, 0.0],
                                                              "ori": [1.0, 0.0, 0.0, 0.0]}
                                          }
                         }
        """
        if 'user_coord' in robot_cmd.keys():
            self.list_to_tuple(robot_cmd)
            logger.info("check user coord : {0}".format(robot_cmd['user_coord']))
            result = self.robot.check_user_coord(robot_cmd['user_coord'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("check user coord : {0}".format(robot_cmd['user_coord']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("check_user_coord bad args, args={0}".format(robot_cmd))

    def _set_relative_offset_on_base(self, robot_cmd):
        """
        * FUNCTION:    _set_relative_offset_on_base
        * DESCRIPTION: 设置基于基座标系运动偏移量
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       relative=(x, y, z) 相对位移，单位(m)
        * EXAMPLE:     {"command": "set_relative_offset_on_base","relative":[0.05,0,0]}
        """
        if 'relative' in robot_cmd.keys():
            logger.info("set relative offset on base :relative {0} ".format(robot_cmd['relative']))
            result = self.robot.set_relative_offset_on_base(robot_cmd['relative'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set relative offset on base :relative {0} user_coord {1}".format(robot_cmd['relative']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_relative_offset_on_base bad args, args={0}".format(robot_cmd))

    def _set_relative_offset_on_user(self, robot_cmd):
        """
        * FUNCTION:    _set_relative_offset_on_user
        * DESCRIPTION: 设置基于用户标系运动偏移量
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       relative=(x, y, z) 相对位移，单位(m)
                       user_coord:用户坐标系
        * EXAMPLE:      {"command": "set_relative_offset_on_user",
                         "relative":[0.05,0,0],
                         "user_coord" : {"coord_type":       2,
                                         "calibrate_method": 0,
                                         "calibrate_points": {"point1": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                                                              "point2": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                                                              "point3": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]},
                                         "tool_desc":       {"pos": [0.0, 0.0, 0.0],
                                                              "ori": [1.0, 0.0, 0.0, 0.0]}
                                          }
                         }
        """
        if 'relative' and 'user_coord' in robot_cmd.keys():
            self.list_to_tuple(robot_cmd)
            logger.info("set relative offset on user : relative {0} user_coord {1}".format(
                robot_cmd['relative'], robot_cmd['user_coord']))
            result = self.robot.set_relative_offset_on_user(robot_cmd['relative'], robot_cmd['user_coord'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set relative offset on user : relative {0} user_coord {1}".format(
                    robot_cmd['relative'], robot_cmd['user_coord']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_relative_offset_on_user bad args, args={0}".format(robot_cmd))

    def _set_no_arrival_ahead(self, robot_cmd):
        """
        * FUNCTION:    _set_no_arrival_ahead
        * DESCRIPTION: 取消提前到位设置
        * INPUTS:
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * EXAMPLE:      {"command":"set_no_arrival_ahead"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.set_no_arrival_ahead()
        if result == RobotErrorType.RobotError_SUCC:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("set no arrival ahead error")

    def _set_arrival_ahead_distance(self, robot_cmd):
        """
        * FUNCTION:    _set_arrival_ahead_distance
        * DESCRIPTION: 设置距离模式下的提前到位距离
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       distance 提前到位距离 单位（米）
        * EXAMPLE:     {"command":"set_arrival_ahead_distance", "distance":0.2}
        """
        if 'distance' in robot_cmd.keys():
            logger.info('set arrival ahead distance : {0}'.format(robot_cmd['distance']))
            result = self.robot.set_arrival_ahead_distance(robot_cmd['distance'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set arrival ahead distance: {0}".format(robot_cmd['distance']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_arrival_ahead_distance bad args, args={0}".format(robot_cmd))

    def _set_arrival_ahead_time(self, robot_cmd):
        """
        * FUNCTION:    _set_arrival_ahead_time
        * DESCRIPTION: 设置时间模式下的提前到位时间
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       sec 提前到位时间　单位（秒）
        * EXAMPLE:     {"command":"set_arrival_ahead_time", "sec":0.2}
        """
        if 'sec' in robot_cmd.keys():
            logger.info('set arrival ahead time : {0}'.format(robot_cmd['sec']))
            result = self.robot.set_arrival_ahead_time(robot_cmd['sec'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set arrival ahead time: {0}".format(robot_cmd['sec']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_arrival_ahead_time bad args, args={0}".format(robot_cmd))

    def _move_track(self, robot_cmd):
        """
        * FUNCTION:    _move_track
        * DESCRIPTION: 轨迹运动
         * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:      track 轨迹类型，包括如下：
        *              圆弧运动RobotMoveTrackType.ARC_CIR
        *              轨迹运动RobotMoveTrackType.CARTESIAN_MOVEP
        * EXAMPLE:     {"command":"move_track", "track":"ARC_CIR"}
        *              {"command":"move_track", "track":"CARTESIAN_MOVEP"}
        """
        if 'track' in robot_cmd.keys():
            logger.info("move track : {0}".format(robot_cmd['track']))
            result = self.robot.move_track(self._tracktype_handler[robot_cmd['track']])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("move track : {0} error".format(robot_cmd['track']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("move_track bad args, args={0}".format(robot_cmd))

    def _move_stop(self, robot_cmd):
        """
        * FUNCTION:    _move_stop
        * DESCRIPTION: 停止机械臂运动
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * EXAMPLE:     {"command":"move_stop"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.move_stop()
        if result == RobotErrorType.RobotError_SUCC:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("move stop error")

    def _move_pause(self, robot_cmd):
        """
        * FUNCTION:    _move_pause
        * DESCRIPTION: 暂停机械臂运动
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * EXAMPLE:     {"command":"move_pause"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.move_pause()
        if result == RobotErrorType.RobotError_SUCC:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("move pause error")

    def _move_continue(self, robot_cmd):
        """
        * FUNCTION:    _move_continue
        * DESCRIPTION: 暂停后回复机械臂运动
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * EXAMPLE:     {"command":"move_continue"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.move_continue()
        if result == RobotErrorType.RobotError_SUCC:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("move continue error")

    def _collision_recover(self, robot_cmd):
        """
        * FUNCTION:    _collision_recover
        * DESCRIPTION: 机械臂碰撞后恢复
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * EXAMPLE:     {"command":"collision_recover"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.collision_recover()
        if result == RobotErrorType.RobotError_SUCC:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("collision recover error")

    def _forward_kin(self, robot_cmd):
        """
        * FUNCTION:    _forward_kin
        * DESCRIPTION: 正解
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: 关节正解结果，结果为详见NOTES
        *              失败返回: None
        * NOTES:       joint_radian:六个关节的关节角，单位(rad) 'joint': [1.0, 1.0, 1.0, 1.0, 1.0, 1.0]
        *              位置 'pos': [-0.06403157614989634, -0.4185973810159096, 0.816883228463401]
        *              姿态 'ori': [-0.11863209307193756, 0.3820514380931854, 0.0, 0.9164950251579285]
        * EXAMPLE:     {"command":"forward_kin","joint_radian":[0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000]}
        """
        if 'joint_radian' in robot_cmd.keys():
            logger.info("forward kin : {0}".format(robot_cmd['joint_radian']))
            result = self.robot.forward_kin(robot_cmd['joint_radian'])
            if result is not None:
                self.send_response(result, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("forward kin : {0} error".format(robot_cmd['joint_radian']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("forward_kin bad args, args={0}".format(robot_cmd))

    def _inverse_kin(self, robot_cmd):
        """
        * FUNCTION:    _inverse_kin
        * DESCRIPTION: 逆解
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: 关节逆解结果，结果为详见NOTES
        *              失败返回: None
        * NOTES:      joint_radian:起始点六个关节的关节角，单位(rad)
        *             pos位置(x, y, z)单位(m)
        *             ori位姿(w, x, y, z)
        * EXAMPLE:     {"command":"inverse_kin",
                        "joint_radian":[1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
                        "pos": [-0.06403157614989634, -0.4185973810159096, 0.816883228463401],
                        "ori": [-0.11863209307193756, 0.3820514380931854, 0.0, 0.9164950251579285]
                        }
        """
        if 'joint_radian' and 'pos' and 'ori' in robot_cmd.keys():
            logger.info("inverse kin: joint_radian {0} pos {1} ori  {2}".format(
                robot_cmd['joint_radian'], robot_cmd['pos'], robot_cmd['ori']))
            result = self.robot.inverse_kin(robot_cmd['joint_radian'], robot_cmd['pos'], robot_cmd['ori'])
            if result is not None:
                self.send_response(result, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("inverse kin: joint_radian {0} pos {1} ori {2}".format(
                    robot_cmd['joint_radian'], robot_cmd['pos'], robot_cmd['ori']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("inverse_kin bad args, args={0}".format(robot_cmd))

    def _base_to_user(self, robot_cmd):
        """
        * FUNCTION:    _base_to_user
        * DESCRIPTION: 用户坐标系转基座坐标系
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: 返回位置和姿态{"pos": (x, y, z), "ori": (w, x, y, z)}
        *              失败返回: None
        * NOTES:       pos:基座标系下的位置(x, y, z)单位(m)
        *              ori:基座标系下的姿态(w, x, y, z)
        *              user_coord:用户坐标系
        *              user_coord = {'coord_type': 2,
        *               'calibrate_method': 0,
        *               'calibrate_points':
        *                   {"point1": (0.0, 0.0, 0.0, 0.0, 0.0, 0.0),
        *                    "point2": (0.0, 0.0, 0.0, 0.0, 0.0, 0.0),
        *                    "point3": (0.0, 0.0, 0.0, 0.0, 0.0, 0.0)},
        *               'tool_desc':
        *                   {"pos": (0.0, 0.0, 0.0),
        *                    "ori": (1.0, 0.0, 0.0, 0.0)}
        *               }
        *               user_tool用户工具描述
        *               user_tool={"pos": (x, y, z), "ori": (w, x, y, z)}
        * EXAMPLE:     {"command": "base_to_user",
                        "pos": [-0.06403157614989634, -0.4185973810159096, 0.816883228463401],
                        "ori": [-0.11863209307193756, 0.3820514380931854, 0.0, 0.9164950251579285],
                        "user_coord" : {"coord_type":       2,
                                        "calibrate_method": 0,
                                        "calibrate_points": {"point1": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                                                              "point2": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                                                              "point3": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]},
                                        "tool_desc":       {"pos": [0.0, 0.0, 0.0],
                                                              "ori": [1.0, 0.0, 0.0, 0.0]}
                                        },
                        "user_tool": {"pos": [0.0, 0.0, 0.0], "ori": [1.0, 0.0, 0.0, 0.0]}
                        }
        """
        if 'pos' and 'ori' and 'user_coord' and 'user_tool' in robot_cmd.keys():
            self.list_to_tuple(robot_cmd)
            logger.info(" base to user: pos {0} ori {1} user_coord {2} user_tool {3}".format(
                robot_cmd['pos'], robot_cmd['ori'], robot_cmd['user_coord'], robot_cmd['user_tool']))
            result = self.robot.base_to_user(robot_cmd['pos'], robot_cmd['ori'], robot_cmd['user_coord'],
                                             robot_cmd['user_tool'])
            if result is not None:
                self.send_response(result, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error(" base to user: pos {0} ori {1} user_coord {2} user_tool {3}".format(
                    robot_cmd['pos'], robot_cmd['ori'], robot_cmd['user_coord'], robot_cmd['user_tool']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("base_to_user bad args, args={0}".format(robot_cmd))

    def _user_to_base(self, robot_cmd):
        """
        * FUNCTION:    _user_to_base
        * DESCRIPTION: 用户坐标系转基座坐标系
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: 返回位置和姿态{"pos": (x, y, z), "ori": (w, x, y, z)}
        *              失败返回: None
        * NOTES:       pos:基座标系下的位置(x, y, z)单位(m)
        *              ori:基座标系下的姿态(w, x, y, z)
        *              user_coord:用户坐标系
        *              user_coord = {'coord_type': 2,
        *               'calibrate_method': 0,
        *               'calibrate_points':
        *                   {"point1": (0.0, 0.0, 0.0, 0.0, 0.0, 0.0),
        *                    "point2": (0.0, 0.0, 0.0, 0.0, 0.0, 0.0),
        *                    "point3": (0.0, 0.0, 0.0, 0.0, 0.0, 0.0)},
        *               'tool_desc':
        *                   {"pos": (0.0, 0.0, 0.0),
        *                    "ori": (1.0, 0.0, 0.0, 0.0)}
        *               }
        *               user_tool用户工具描述
        *               user_tool={"pos": (x, y, z), "ori": (w, x, y, z)}
        * EXAMPLE:     {"command": "user_to_base",
                        "pos": [-0.06403157614989634, -0.4185973810159096, 0.816883228463401],
                        "ori": [-0.11863209307193756, 0.3820514380931854, 0.0, 0.9164950251579285],
                        "user_coord" : {"coord_type":       2,
                                        "calibrate_method": 0,
                                        "calibrate_points": {"point1": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                                                              "point2": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                                                              "point3": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]},
                                        "tool_desc":       {"pos": [0.0, 0.0, 0.0],
                                                              "ori": [1.0, 0.0, 0.0, 0.0]}
                                        },
                        "user_tool": {"pos": [0.0, 0.0, 0.0], "ori": [1.0, 0.0, 0.0, 0.0]}
                        }
        """
        if 'pos' and 'ori' and 'user_coord' and 'user_tool' in robot_cmd.keys():
            self.list_to_tuple(robot_cmd)
            logger.info(" user to base: pos {0} ori {1} user_coord {2} user_tool {3}".format(
                robot_cmd['pos'], robot_cmd['ori'], robot_cmd['user_coord'], robot_cmd['user_tool']))
            result = self.robot.user_to_base(robot_cmd['pos'], robot_cmd['ori'], robot_cmd['user_coord'],
                                             robot_cmd['user_tool'])
            if result is not None:
                self.send_response(result, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("user to base: pos {0} ori {1} user_coord {2} user_tool {3}".format(
                    robot_cmd['pos'], robot_cmd['ori'], robot_cmd['user_coord'], robot_cmd['user_tool']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("user_to_base bad args, args={0}".format(robot_cmd))

    def _base_to_base_additional_tool(self, robot_cmd):
        """
        * FUNCTION:    _base_to_base_additional_tool
        * DESCRIPTION: 基坐标系转基座标得到工具末端点的位置和姿态
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: 返回基于基座标系的工具末端位置位置和姿态信息{"pos": (x, y, z), "ori": (w, x, y, z)}
        *              失败返回: None
        * NOTES:       pos:基于基座标系的法兰盘中心位置信息(x, y, z)单位(m)
        *              ori:基于基座标系的姿态信息(w, x, y, z)
        *              user_tool用户工具描述
        *              user_tool={"pos": (x, y, z), "ori": (w, x, y, z)}
        * EXAMPLE:     {"command": "base_to_base_additional_tool",
                        "pos": [-0.06403157614989634, -0.4185973810159096, 0.816883228463401],
                        "ori": [-0.11863209307193756, 0.3820514380931854, 0.0, 0.9164950251579285],
                        "user_tool": {"pos": [0.0, 0.0, 0.0], "ori": [1.0, 0.0, 0.0, 0.0]}
                        }
        """
        if 'pos' and 'ori' and 'user_tool' in robot_cmd.keys():
            self.list_to_tuple(robot_cmd)
            logger.info(" base to base additional tool: pos {0} ori {1} user_tool {2}".format(
                robot_cmd['pos'], robot_cmd['ori'], robot_cmd['user_tool']))
            result = self.robot.base_to_base_additional_tool(robot_cmd['pos'], robot_cmd['ori'], robot_cmd['user_tool'])
            if result is not None:
                self.send_response(result, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error(" base to base additional tool: pos {0} ori {1} user_tool {2}".format(
                    robot_cmd['pos'], robot_cmd['ori'], robot_cmd['user_tool']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("base_to_base_additional_tool bad args, args={0}".format(robot_cmd))

    def _rpy_to_quaternion(self, robot_cmd):
        """
        * FUNCTION:    _rpy_to_quaternion
        * DESCRIPTION: 欧拉角转四元数
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: 四元数结果，结果为详见NOTES
        *              失败返回: None
        * NOTES:       四元素(w, x, y, z)
        *              rpy:欧拉角(rx, ry, rz)，单位(m)
        * EXAMPLE:     {"command": "rpy_to_quaternion",
                        "rpy": [0.0,0.0,0.0]
                        }
        """
        if 'rpy' in robot_cmd.keys():
            self.list_to_tuple(robot_cmd)
            logger.info(" rpy to quaternion ， rpy ：{0} ".format(robot_cmd['rpy']))
            result = self.robot.rpy_to_quaternion(robot_cmd['rpy'])
            if result is not None:
                self.send_response(result, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error(" rpy to quaternion ， rpy ：{0} ".format(robot_cmd['rpy']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("rpy_to_quaternion bad args, args={0}".format(robot_cmd))

    def _quaternion_to_rpy(self, robot_cmd):
        """
        * FUNCTION:    _quaternion_to_rpy
        * DESCRIPTION: 四元数转欧拉角
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: 欧拉角结果，结果为详见NOTES
        *              失败返回: None
        * NOTES:       rpy:欧拉角(rx, ry, rz)，单位(m)
        *              四元数(w, x, y, z)
        * EXAMPLE:     {"command": "quaternion_to_rpy",
                        "ori": [-0.11863209307193756, 0.3820514380931854, 0.0, 0.9164950251579285]
                        }
        """
        if 'ori' in robot_cmd.keys():
            self.list_to_tuple(robot_cmd)
            logger.info(" quaternion to rpy ， ori ：{0} ".format(robot_cmd['ori']))
            result = self.robot.quaternion_to_rpy(robot_cmd['ori'])
            if result is not None:
                self.send_response(result, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error(" quaternion to rpy ， ori ：{0} ".format(robot_cmd['ori']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("quaternion_to_rpy bad args, args={0}".format(robot_cmd))

    def _set_none_tool_kinematics_param(self, robot_cmd):
        """
        * FUNCTION:     _set_none_tool_kinematics_param
        * DESCRIPTION:  设置无工具运动学参数　
        * INPUTS:
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: None
        * EXAMPLE:     {"command":"set_none_tool_kinematics_param"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.set_none_tool_kinematics_param()
        if result is not None:
            self.send_response(result, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("set none tool kinematics param")

    def _set_tool_kinematics_param(self, robot_cmd):
        """
        * FUNCTION:    _set_tool_kinematics_param
        * DESCRIPTION: 设置工具的运动学参数　
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       末端工具参数： tool_end_param={"pos": (x, y, z), "ori": (w, x, y, z)}
        * EXAMPLE:     {"command": "set_tool_kinematics_param",
                        "tool_end_param":{
                                            "pos": [-0.06403157614989634, -0.4185973810159096, 0.816883228463401],
                                            "ori": [-0.11863209307193756, 0.3820514380931854, 0.0, 0.9164950251579285]
                                            }
                        }
        """
        if 'tool_end_param' in robot_cmd.keys():
            self.list_to_tuple(robot_cmd)
            logger.info(" set tool kinematics param: tool_end_param {0}".format(robot_cmd['tool_end_param']))
            result = self.robot.set_tool_kinematics_param(robot_cmd['tool_end_param'])
            if result is not None:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error(" set tool kinematics param: tool_end_param {0}".format(robot_cmd['tool_end_param']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_tool_kinematics_param bad args, args={0}".format(robot_cmd))

    def _get_tool_kinematics_param(self, robot_cmd):
        """
        * FUNCTION:     _get_tool_kinematics_param
        * DESCRIPTION:  获取工具的运动学参数　
        * INPUTS:
        * RETURNS:     成功返回: 工具的运动学参数
        *              失败返回: None
        * NOTES:       tool_end_param={"pos": (x, y, z), "ori": (w, x, y, z)}
        * EXAMPLE:     {"command":"get_tool_kinematics_param"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.get_tool_kinematics_param()
        if result is not None:
            self.send_response(result, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("get tool kinematics param")

    def _set_none_tool_dynamics_param(self, robot_cmd):
        """
        * FUNCTION:     _set_none_tool_dynamics_param
        * DESCRIPTION:  设置无工具动力学参数　
        * INPUTS:
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: None
        * EXAMPLE:     {"command":"set_none_tool_dynamics_param"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.set_none_tool_dynamics_param()
        if result is not None:
            self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("set none tool dynamics param")

    def _set_tool_dynamics_param(self, robot_cmd):
        """
        * FUNCTION:    _set_tool_dynamics_param
        * DESCRIPTION: 设置工具的动力学参数　
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:      tool_dynamics: 动力学参数
        *              tool_dynamics = 位置，单位(m) ：{"position": [0.0, 0.0, 0.0],
        *                              负载，单位(kg)： "payload": 1.0,
        *                              惯量：          "inertia": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]}
        * EXAMPLE:     {"command": "set_tool_dynamics_param",
                        "tool_dynamics":{
                                            "position": [0.0, 0.0, 0.0],
                                            "payload": 1.0,
                                            "inertia": [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
                                            }
                        }
        """
        if 'tool_dynamics' in robot_cmd.keys():
            self.list_to_tuple(robot_cmd)
            logger.info(" set tool dynamics param ， tool_dynamics ：{0}".format(robot_cmd['tool_dynamics']))
            result = self.robot.set_tool_dynamics_param(robot_cmd['tool_dynamics'])
            if result is not None:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error(" set tool dynamics param: tool_dynamics {0}".format(robot_cmd['tool_dynamics']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_tool_dynamics_param bad args, args={0}".format(robot_cmd))

    def _get_tool_dynamics_param(self, robot_cmd):
        """
        * FUNCTION:     _get_tool_dynamics_param
        * DESCRIPTION:  获取工具的动力学参数　
        * INPUTS:
        * RETURNS:     成功返回: 运动学参数
        *              tool_dynamics = 位置，单位(m) ：{"position": (0.0, 0.0, 0.0),
        *                              负载，单位(kg)： "payload": 1.0,
        *                              惯量：          "inertia": (0.0, 0.0, 0.0, 0.0, 0.0, 0.0)}
        *              失败返回: None
        * EXAMPLE:     {"command":"get_tool_dynamics_param"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.get_tool_dynamics_param()
        if result is not None:
            self.send_response(result, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("get tool dynamics param")

    def _get_robot_state(self, robot_cmd):
        """
        * FUNCTION:     _get_robot_state
        * DESCRIPTION:  获取机械臂当前状态
        * INPUTS:
        * RETURNS:     成功返回: 机械臂当前状态
        *              机械臂当前停止:RobotStatus.Stopped : 0
        *              机械臂当前运行:RobotStatus.Running : 1
        *              机械臂当前暂停:RobotStatus.Paused : 2
        *              机械臂当前恢复:RobotStatus.Resumed : 3
        *              失败返回: None
        * EXAMPLE:     {"command":"get_robot_state"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.get_robot_state()
        if result is not None:
            self.send_response(RobotStatusResponse(result).get_response(), RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("get robot state")

    def _get_work_mode(self, robot_cmd):
        """
        * FUNCTION:     _get_work_mode
        * DESCRIPTION:  获取机械臂服务器当前工作模式
        * INPUTS:
        * RETURNS:     成功返回: 服务器工作模式
        *              机械臂仿真模式:RobotRunningMode.RobotModeSimulator
        *              机械臂真实模式:RobotRunningMode.RobotModeReal
        *              失败返回: None
        * EXAMPLE:     {"command":"get_work_mode"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.get_work_mode()
        if result is not None:
            self.send_response(RobotRunningModeResponse(result).get_response(), RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("get work mode")

    def _set_work_mode(self, robot_cmd):
        """
        * FUNCTION:    _set_work_mode
        * DESCRIPTION: 设置机械臂服务器工作模式
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       mode:服务器工作模式
        *              机械臂仿真模式:RobotRunningMode.RobotModeSimulator
        *              机械臂真实模式:RobotRunningMode.RobotModeReal
        * EXAMPLE:     {"command":"set_work_mode", "mode":0}
        """
        if 'mode' in robot_cmd.keys():
            logger.info('set work mode: {0}'.format(robot_cmd['mode']))
            result = self.robot.set_work_mode(robot_cmd['mode'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set work mode: {0}".format(robot_cmd['mode']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_work_mode bad args, args={0}".format(robot_cmd))

    def _set_collision_class(self, robot_cmd):
        """
        * FUNCTION:    _set_collision_class
        * DESCRIPTION: 设置机械臂碰撞等级
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * NOTES:       grade碰撞等级:碰撞等级 范围（0～10）
        * EXAMPLE:     {"command":"set_collision_class", "grade":6}
        """
        if 'grade' in robot_cmd.keys():
            logger.info('set collision class : {0}'.format(robot_cmd['grade']))
            result = self.robot.set_collision_class(robot_cmd['grade'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set collision class: {0}".format(robot_cmd['grade']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_collision_class bad args, args={0}".format(robot_cmd))

    def _is_have_real_robot(self, robot_cmd):
        """
        * FUNCTION:     _is_have_real_robot
        * DESCRIPTION:  获取当前是否已经链接真实机械臂
        * INPUTS:
        * RETURNS:     成功返回: 1：存在 0：不存在
        *              失败返回: 其他
        * EXAMPLE:     {"command":"is_have_real_robot"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.is_have_real_robot()
        if result is 0 or 1:
            self.send_response(result, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("is have real robot")

    def _is_online_master_mode(self, robot_cmd):
        """
        * FUNCTION:     _is_online_master_mode
        * DESCRIPTION:  当前机械臂是否运行在联机主模式
        * INPUTS:
        * RETURNS:     成功返回: 1：主模式 0：从模式
        *              失败返回: 其他
        * EXAMPLE:     {"command":"is_online_master_mode"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.is_online_master_mode()
        if result is 0 or 1:
            self.send_response(result, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("is online master mode")

    def _get_joint_status(self, robot_cmd):
        """
        * FUNCTION:     _get_joint_status
        * DESCRIPTION:  获取机械臂当前状态信息
        * INPUTS:
        * RETURNS:     成功返回: 返回六个关节状态，包括：电流，电压，温度
        *              {'joint1': {'current': 电流(毫安), 'voltage': 电压(伏特), 'temperature': 温度(摄氏度)},
        *              'joint2': {'current': 0, 'voltage': 0.0, 'temperature': 0},
        *              'joint3': {'current': 0, 'voltage': 0.0, 'temperature': 0},
        *              'joint4': {'current': 0, 'voltage': 0.0, 'temperature': 0},
        *              'joint5': {'current': 0, 'voltage': 0.0, 'temperature': 0},
        *              'joint6': {'current': 0, 'voltage': 0.0, 'temperature': 0}}
        *              失败返回: None
        * EXAMPLE:     {"command":"get_joint_status"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.get_joint_status()
        if result is not None:
            self.send_response(result, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("get joint status")

    def _get_board_io_config(self, robot_cmd):
        """
        * FUNCTION:    _get_board_io_config
        * DESCRIPTION: 获取控制柜IO和用户IO配置信息
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: IO配置
        *               [{"id": ID
        *                 "name": "IO名字"
        *                 "addr": IO地址
        *                 "type": IO类型
        *                 "value": IO当前值},]
        *              失败返回: None
        * NOTES:       io_type：IO类型：RobotIOType
        *               # 控制柜IO
        *               ControlBox_DI = 0
        *               ControlBox_DO = 1
        *               ControlBox_AI = 2
        *               ControlBox_AO = 3
        *               # 用户IO
        *               User_DI = 4
        *               User_DO = 5
        *               User_AI = 6
        *               User_AO = 7
        * EXAMPLE:     {"command":"get_board_io_config", "io_type":0}
        """
        if 'io_type' in robot_cmd.keys():
            logger.info('get board io config : {0}'.format(robot_cmd['io_type']))
            result = self.robot.get_board_io_config(robot_cmd['io_type'])
            if result is not None:
                self.send_response(result, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("get board io config: {0}".format(robot_cmd['io_type']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("get_board_io_config bad args, args={0}".format(robot_cmd))

    def _get_board_io_status(self, robot_cmd):
        """
        * FUNCTION:    _get_board_io_status
        * DESCRIPTION: 获取IO状态
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: IO状态 double数值(数字IO，返回0或1,模拟IO返回浮点数）
        *              失败返回: None
        * NOTES:       io_type:类型
        *              io_name:名称
        * EXAMPLE:     {"command":"get_board_io_status", "io_type":0, "io_name":"SI10"}
        """
        if 'io_type' and 'io_name' in robot_cmd.keys():
            logger.info(
                'get board io status, io_type: {0} io_name {1}'.format(robot_cmd['io_type'], robot_cmd['io_name']))
            result = self.robot.get_board_io_status(robot_cmd['io_type'], robot_cmd['io_name'])
            if result is not None:
                self.send_response(result, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error('get board io status: io_type {0} io_name {1}'.format(robot_cmd['io_type'],
                                                                                    robot_cmd['io_name']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("get_board_io_status bad args, args={0}".format(robot_cmd))

    def _set_board_io_status(self, robot_cmd):
        """
        * FUNCTION:    _set_board_io_status
        * DESCRIPTION: 设置IO状态
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其它
        * NOTES:       io_type:类型
        *              io_name:名称
        *              io_value:状态数值(数字IO，返回0或1,模拟IO返回浮点数）
        * EXAMPLE:     {"command":"set_board_io_status", "io_type":0, "io_name":"SI10","io_value":1}
        """
        if 'io_type' and 'io_name' and 'io_value' in robot_cmd.keys():
            logger.info('set board io status: io_type {0} io_name {1} io_value {2}'.format(
                robot_cmd['io_type'], robot_cmd['io_name'], robot_cmd['io_value']))
            result = self.robot.set_board_io_status(robot_cmd['io_type'], robot_cmd['io_name'], robot_cmd['io_value'])
            if result == RobotJasonServerError.RESUTL_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error('set board io status, io_type: {0} io_name {1} io_value {2}'.format(
                    robot_cmd['io_type'], robot_cmd['io_name'], robot_cmd['io_value']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_board_io_status bad args, args={0}".format(robot_cmd))

    def _set_tool_power_type(self, robot_cmd):
        """
        * FUNCTION:    _set_tool_power_type
        * DESCRIPTION: 设置工具端电源类型
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其他
        * INPUTS:      power_type:电源类型
        *              RobotToolPowerType.OUT_0V : 0
        *              RobotToolPowerType.OUT_12V : 1
        *              RobotToolPowerType.OUT_24V : 2
        * EXAMPLE:     {"command":"set_tool_power_type", "power_type":0}
        """
        if 'power_type' in robot_cmd.keys():
            logger.info('set_tool_power_type: {0}'.format(robot_cmd['power_type']))
            result = self.robot.set_tool_power_type(robot_cmd['power_type'])
            if result == RobotErrorType.RobotError_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("set_tool_power_type: {0}".format(robot_cmd['power_type']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_tool_power_type bad args, args={0}".format(robot_cmd))

    def _get_tool_power_type(self, robot_cmd):
        """
        * FUNCTION:     _get_tool_power_type
        * DESCRIPTION:  获取工具端电源类型
        * INPUTS:
        * RETURNS:     成功返回: 电源类型，包括如下：
        *              RobotToolPowerType.OUT_0V : 0
        *              RobotToolPowerType.OUT_12V : 1
        *              RobotToolPowerType.OUT_24V : 2
        *              失败返回: None
        * NOTES:       power_type:电源类型
        * EXAMPLE:     {"command":"get_tool_power_type"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.get_tool_power_type()
        if result is not None:
            self.send_response(result, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("get tool power type")

    def _set_tool_io_type(self, robot_cmd):
        """
        * FUNCTION:    _set_tool_io_type
        * DESCRIPTION: 设置工具端数字IO类型
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: IO类型，包括如下：
        *                       RobotToolDigitalIoDir.IO_IN
        *                       RobotToolDigitalIoDir.IO_OUT
        *              失败返回: None
        * NOTES:       io_addr:工具端IO地址 详见class RobotToolIoAddr
        *               TOOL_DIGITAL_IO_0 = 0
        *               TOOL_DIGITAL_IO_1 = 1
        *               TOOL_DIGITAL_IO_2 = 2
        *               TOOL_DIGITAL_IO_3 = 3
        *              io_type:工具端IO类型 详见class RobotToolDigitalIoDir
        *               # 输入 : IO_IN = 0
        *              # 输出 : IO_OUT = 1
        * EXAMPLE:     {"command":"set_tool_io_type", "io_addr":0, "io_type":1}
        """
        if 'io_addr' and 'io_type' in robot_cmd.keys():
            logger.info('set tool io type: io_addr {0} io_type {1}'.format(
                robot_cmd['io_addr'], robot_cmd['io_type']))
            result = self.robot.set_tool_io_type(robot_cmd['io_addr'], robot_cmd['io_type'])
            if result is not None:
                self.send_response(result, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error('set tool io type: io_addr {0} io_type {1}'.format(
                    robot_cmd['io_addr'], robot_cmd['io_type']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_tool_io_type bad args, args={0}".format(robot_cmd))

    def _get_tool_power_voltage(self, robot_cmd):
        """
        * FUNCTION:     _get_tool_power_voltage
        * DESCRIPTION:  获取工具端电压数值
        * INPUTS:
        * RETURNS:     成功返回: 返回电压数值，单位（伏特）
        *              失败返回: None
        * EXAMPLE:     {"command":"get_tool_power_voltage"}
        """
        logger.info("{0}".format(robot_cmd["command"]))
        result = self.robot.get_tool_power_voltage()
        if result is not None:
            self.send_response(result, RobotResponse.succ)
        else:
            self.send_response(result, RobotResponse.failed)
            logger.error("get tool power voltage")

    def _get_tool_io_status(self, robot_cmd):
        """
        * FUNCTION:    _get_tool_io_status
        * DESCRIPTION: 获取工具端IO状态
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: 返回工具端IO状态
        *              失败返回: None
        * NOTES:       io_name:IO名称
        * EXAMPLE:     {"command":"get_tool_io_status", "io_name":0}
        """
        if 'io_name' in robot_cmd.keys():
            logger.info('get tool io status: {0}'.format(robot_cmd['io_name']))
            result = self.robot.get_tool_io_status(robot_cmd['io_name'])
            if result is not None:
                self.send_response(result, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error("get tool io status: {0}".format(robot_cmd['io_name']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("get_tool_io_status bad args, args={0}".format(robot_cmd))

    def _set_tool_io_status(self, robot_cmd):
        """
        * FUNCTION:    _set_tool_io_status
        * DESCRIPTION: 设置工具端IO状态
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其它
        * NOTES:       io_name：工具端IO名称
        *              io_status:工具端IO状态: 取值范围（0或1）
        * EXAMPLE:     {"command":"set_tool_io_status", "io_name":0, "io_status":0}
        """
        if 'io_name' and 'io_status' in robot_cmd.keys():
            logger.info('set tool io status: io_name {0} io_status {1}'.format(
                robot_cmd['io_name'], robot_cmd['io_status']))
            result = self.robot.set_tool_io_status(robot_cmd['io_name'], robot_cmd['io_status'])
            if result == RobotJasonServerError.RESUTL_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error('set tool io status: io_name {0} io_status {1}'.format(
                    robot_cmd['io_name'], robot_cmd['io_status']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error("set_tool_io_status bad args, args={0}".format(robot_cmd))

    def _connect_to_modbus_server(self, robot_cmd):
        """
        * FUNCTION:    _connect_to_modbus_server
        * DESCRIPTION: 连接到Modbus服务端
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其它
        * NOTES:       mode：modbus工作模式：tcp  rtu
        *              method:rtu模式下的两种通信方式: rtu  ascii
        *              host:TCP模式下为服务端IP地址
        *                   RTU模式下为服务端端口名称
        * EXAMPLE:     {"command":"connect_to_modbus_server","mode":"tcp","host":"192.168.1.30","port":502,"timeout":3}
        *              {"command":"connect_to_modbus_server","mode":"rtu","method":"rtu","host":"/dev/ttyUSB0","stopbits":1,"bytesize":8,"parity":"N","baudrate":115200,"timeout":3}
        *              {"command":"connect_to_modbus_server","mode":"rtu","method":"ascii","host":"/dev/ttyUSB0","stopbits":1,"bytesize":8,"parity":'N',"baudrate":115200,"timeout":3}
        """
        if robot_cmd['host'] in self.robot_modbus_client:
            logger.error("Connected to modbus server repeatedly:{0}".format(robot_cmd['host']))
            self.send_response(RobotJasonServerError.RESUTL_ERROR_CONNECT_TO_MODBUS_SERVER, RobotResponse.failed)
            return None
        if 'mode' in robot_cmd.keys() and robot_cmd['mode'] in ('tcp', 'rtu'):
            logger.info('connect to modbus server: {0}'.format(robot_cmd))
            robot_cmd.pop('command')
            self.robot_modbus_client[robot_cmd["host"]] = RobotModbusClient()
            result = self.robot_modbus_client[robot_cmd["host"]].connect_to_modbus_server(**robot_cmd)
            if result == RobotJasonServerError.RESUTL_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.robot_modbus_client.pop(robot_cmd["host"])
                self.send_response(result, RobotResponse.failed)
                logger.error('connect to modbus server failed: {0}'.format(robot_cmd))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error('connect_to_modbus_server bad args: args={0}'.format(robot_cmd))

    def _modbus_client_read_coils(self, robot_cmd):
        """
        * FUNCTION:    _modbus_client_read_coils
        * DESCRIPTION: 读线圈
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: modbus服务端线圈状态
        *              失败返回: 其它
        * NOTES:       address: 线圈起始地址
        *              count: 读取线圈数量
        *              unit: modbus server端编码ID
        * EXAMPLE:     {"command":"modbus_client_read_coils","host":"192.168.1.30","address":512,"count":2, "unit":1}
        *              {"command":"modbus_client_read_coils","host":"/dev/ttyUSB0","address":512,"count":2, "unit":1}
        """
        if robot_cmd['host'] not in self.robot_modbus_client:
            logger.error("Not connected to modbus server:{0}".format(robot_cmd['host']))
            self.send_response(RobotJasonServerError.RESUTL_ERROR_DISCONNECT_TO_MODBUS_SERVER, RobotResponse.failed)
            return None
        if 'address' and 'count' and 'unit' in robot_cmd.keys():
            logger.info('modbus client read coils: host{0} address {1} count {2} unit {3}'.format(
                robot_cmd['host'], robot_cmd['address'], robot_cmd['count'], robot_cmd['unit']))
            result = self.robot_modbus_client[robot_cmd["host"]].modbus_client_read_coils(robot_cmd['address'], robot_cmd['count'], robot_cmd['unit'])
            if result is not None:
                self.send_response(result, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error('modbus client read coils failed: host{0} address {1} count {2} unit {3}'.format(
                    robot_cmd['host'], robot_cmd['address'], robot_cmd['count'], robot_cmd['unit']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error('modbus_client_read_coils bad args: args={0}'.format(robot_cmd))

    def _modbus_client_read_discrete_inputs(self, robot_cmd):
        """
        * FUNCTION:    _modbus_client_read_discrete_inputs
        * DESCRIPTION: 读离散量
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: modbus服务端离散量状态
        *              失败返回: 其它
        * NOTES:       address: 离散量起始地址
        *              count: 读取离散量数量
        *              unit: modbus server端编码ID
        * EXAMPLE:     {"command":"modbus_client_read_discrete_inputs","host":"192.168.1.30","address":0,"count":2, "unit":1}
        *              {"command":"modbus_client_read_discrete_inputs","host":"/dev/ttyUSB0","address":0,"count":2, "unit":1}
        """
        if robot_cmd['host'] not in self.robot_modbus_client:
            logger.error("Not connected to modbus server:{0}".format(robot_cmd['host']))
            self.send_response(RobotJasonServerError.RESUTL_ERROR_DISCONNECT_TO_MODBUS_SERVER, RobotResponse.failed)
            return None
        if 'address' and 'count' and 'unit' in robot_cmd.keys():
            logger.info('modbus client read discrete inputs: host{0} address {1} count {2} unit {3}'.format(
                robot_cmd['host'], robot_cmd['address'], robot_cmd['count'], robot_cmd['unit']))
            result = self.robot_modbus_client[robot_cmd["host"]].modbus_client_read_discrete_inputs(robot_cmd['address'], robot_cmd['count'], robot_cmd['unit'])
            if result is not None:
                self.send_response(result, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error('modbus client read discrete inputs failed: host{0} address {1} count {2} unit {3}'.format(
                    robot_cmd['host'], robot_cmd['address'], robot_cmd['count'], robot_cmd['unit']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error('modbus_client_read_discrete_inputs bad args: args={0}'.format(robot_cmd))

    def _modbus_client_read_holding_registers(self, robot_cmd):
        """
        * FUNCTION:    _modbus_client_read_holding_registers
        * DESCRIPTION: 读保持寄存器
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: modbus服务端保持寄存器数值
        *              失败返回: 其它
        * NOTES:       address: 保持寄存器起始地址
        *              count: 读取保持寄存器数量
        *              unit: modbus server端编码ID
        * EXAMPLE:     {"command":"modbus_client_read_holding_registers","host":"192.168.1.30","address":124, "count":2, "unit":1}
        *              {"command":"modbus_client_read_holding_registers","host":"/dev/ttyUSB0","address":124, "count":2, "unit":1}
        """
        if robot_cmd['host'] not in self.robot_modbus_client:
            logger.error("Not connected to modbus server:{0}".format(robot_cmd['host']))
            self.send_response(RobotJasonServerError.RESUTL_ERROR_DISCONNECT_TO_MODBUS_SERVER, RobotResponse.failed)
            return None
        if 'address' and 'count' and 'unit' in robot_cmd.keys():
            logger.info('modbus client read holding registers: host{0} address {1} count {2} unit {3}'.format(
                robot_cmd['host'], robot_cmd['address'], robot_cmd['count'], robot_cmd['unit']))
            result = self.robot_modbus_client[robot_cmd["host"]].modbus_client_read_holding_registers(robot_cmd['address'], robot_cmd['count'], robot_cmd['unit'])
            if result is not None:
                self.send_response(result, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error('modbus client read holding registers failed: host{0} address {1} count {2} unit {3}'.format(
                    robot_cmd['host'], robot_cmd['address'], robot_cmd['count'], robot_cmd['unit']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error('modbus_client_read_holding_registers bad args: args={0}'.format(robot_cmd))

    def _modbus_client_read_input_registers(self, robot_cmd):
        """
        * FUNCTION:    _modbus_client_read_input_registers
        * DESCRIPTION: 读保持寄存器
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: modbus服务端输入寄存器数值
        *              失败返回: 其它
        * NOTES:       address: 输入寄存器起始地址
        *              count: 读取输入寄存器数量
        *              unit: modbus server端编码ID
        * EXAMPLE:     {"command":"modbus_client_read_input_registers","host":"192.168.1.30","address":64, "count":8, "unit":1}
        *              {"command":"modbus_client_read_input_registers","host":"/dev/ttyUSB0","address":1000, "count":1, "unit":1}
        """
        if robot_cmd['host'] not in self.robot_modbus_client:
            logger.error("Not connected to modbus server:{0}".format(robot_cmd['host']))
            self.send_response(RobotJasonServerError.RESUTL_ERROR_DISCONNECT_TO_MODBUS_SERVER, RobotResponse.failed)
            return None
        if 'address' and 'count' and 'unit' in robot_cmd.keys():
            logger.info('modbus client read input registers: host{0} address {1} count {2} unit {3}'.format(
                robot_cmd['host'], robot_cmd['address'], robot_cmd['count'], robot_cmd['unit']))
            result = self.robot_modbus_client[robot_cmd["host"]].modbus_client_read_input_registers(robot_cmd['address'], robot_cmd['count'], robot_cmd['unit'])
            if result is not None:
                self.send_response(result, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error('modbus client read input registers failed: host{0} address {1} count {2} unit {3}'.format(
                    robot_cmd['host'], robot_cmd['address'], robot_cmd['count'], robot_cmd['unit']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error('modbus_client_read_input_registers bad args: args={0}'.format(robot_cmd))

    def _modbus_client_write_coil(self, robot_cmd):
        """
        * FUNCTION:    _modbus_client_write_coil
        * DESCRIPTION: 写单个线圈
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其它
        * NOTES:       address: 线圈起始地址
        *              value: 写入线圈数值
        *              unit: modbus server端编码ID
        * EXAMPLE:     {"command":"modbus_client_write_coil","host":"192.168.1.30","address":512, "value":0, "unit":1}
        *              {"command":"modbus_client_write_coil","host":"/dev/ttyUSB0","address":1000, "count":1, "unit":1}
        """
        if robot_cmd['host'] not in self.robot_modbus_client:
            logger.error("Not connected to modbus server:{0}".format(robot_cmd['host']))
            self.send_response(RobotJasonServerError.RESUTL_ERROR_DISCONNECT_TO_MODBUS_SERVER, RobotResponse.failed)
            return None
        if 'address' and 'value' and 'unit' in robot_cmd.keys():
            logger.info('modbus client write coil: host{0} address {1} value {2} unit {3}'.format(
                robot_cmd['host'], robot_cmd['address'], robot_cmd['value'], robot_cmd['unit']))
            result = self.robot_modbus_client[robot_cmd["host"]].modbus_client_write_coil(robot_cmd['address'], robot_cmd['value'], robot_cmd['unit'])
            if result == RobotJasonServerError.RESUTL_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error('modbus client write coil failed: host{0} address {1} value {2} unit {3}'.format(
                    robot_cmd['host'], robot_cmd['address'], robot_cmd['value'], robot_cmd['unit']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error('modbus_client_write_coil bad args: args={0}'.format(robot_cmd))

    def _modbus_client_write_register(self, robot_cmd):
        """
        * FUNCTION:    _modbus_client_write_register
        * DESCRIPTION: 写单个寄存器
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其它
        * NOTES:       address: 寄存器起始地址
        *              value: 写入寄存器数值
        *              unit: modbus server端编码ID
        * EXAMPLE:     {"command":"modbus_client_write_register","host":"192.168.1.30","address":124, "value":30, "unit":1}
        *              {"command":"modbus_client_write_register","host":"/dev/ttyUSB0","address":1000, "count":1, "unit":1}
        """
        if robot_cmd['host'] not in self.robot_modbus_client:
            logger.error("Not connected to modbus server:{0}".format(robot_cmd['host']))
            self.send_response(RobotJasonServerError.RESUTL_ERROR_DISCONNECT_TO_MODBUS_SERVER, RobotResponse.failed)
            return None
        if 'address' and 'value' and 'unit' in robot_cmd.keys():
            logger.info('modbus client write register: host{0} address {1} value {2} unit {3}'.format(
                robot_cmd['host'], robot_cmd['address'], robot_cmd['value'], robot_cmd['unit']))
            result = self.robot_modbus_client[robot_cmd["host"]].modbus_client_write_registers(robot_cmd['address'], robot_cmd['value'], robot_cmd['unit'])
            if result == RobotJasonServerError.RESUTL_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error('modbus client write register failed: host{0} address {1} value {2} unit {3}'.format(
                    robot_cmd['host'], robot_cmd['address'], robot_cmd['value'], robot_cmd['unit']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error('modbus_client_write_register bad args: args={0}'.format(robot_cmd))

    def _modbus_client_write_coils(self, robot_cmd):
        """
        * FUNCTION:    _modbus_client_write_coils
        * DESCRIPTION: 写多个线圈
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其它
        * NOTES:       address: 线圈起始地址
        *              values: 写入线圈数值
        *              unit: modbus server端编码ID
        * EXAMPLE:     {"command":"modbus_client_write_coils","host":"192.168.1.30","address":512, "values":[0,0], "unit":1}
        *              {"command":"modbus_client_write_coils","host":"/dev/ttyUSB0","address":1000, "count":1, "unit":1}
        """
        if robot_cmd['host'] not in self.robot_modbus_client:
            logger.error("Not connected to modbus server:{0}".format(robot_cmd['host']))
            self.send_response(RobotJasonServerError.RESUTL_ERROR_DISCONNECT_TO_MODBUS_SERVER, RobotResponse.failed)
            return None
        if 'address' and 'values' and 'unit' in robot_cmd.keys():
            logger.info('modbus client write coils: host{0} address {1} value {2} unit {3}'.format(
                robot_cmd['host'], robot_cmd['address'], robot_cmd['values'], robot_cmd['unit']))
            result = self.robot_modbus_client[robot_cmd["host"]].modbus_client_write_coils(robot_cmd['address'], robot_cmd['values'], robot_cmd['unit'])
            if result == RobotJasonServerError.RESUTL_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error('modbus client write coils failed: host{0} address {1} value {2} unit {3}'.format(
                    robot_cmd['host'], robot_cmd['address'], robot_cmd['values'], robot_cmd['unit']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error('modbus_client_write_coils bad args: args={0}'.format(robot_cmd))

    def _modbus_client_write_registers(self, robot_cmd):
        """
        * FUNCTION:    _modbus_client_write_registers
        * DESCRIPTION: 写多个寄存器
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其它
        * NOTES:       address: 寄存器起始地址
        *              values: 写入寄存器数值
        *              unit: modbus server端编码ID
        * EXAMPLE:     {"command":"modbus_client_write_registers","host":"192.168.1.30","address":124, "values":[10,10], "unit":1}
        *              {"command":"modbus_client_write_registers","host":"/dev/ttyUSB0","address":1000, "count":1, "unit":1}
        """
        if robot_cmd['host'] not in self.robot_modbus_client:
            logger.error("Not connected to modbus server:{0}".format(robot_cmd['host']))
            self.send_response(RobotJasonServerError.RESUTL_ERROR_DISCONNECT_TO_MODBUS_SERVER, RobotResponse.failed)
            return None
        if 'address' and 'values' and 'unit' in robot_cmd.keys():
            logger.info('modbus client write registers: host{0} address {1} value {2} unit {3}'.format(
                robot_cmd['host'], robot_cmd['address'], robot_cmd['values'], robot_cmd['unit']))
            result = self.robot_modbus_client[robot_cmd["host"]].modbus_client_write_registers(robot_cmd['address'], robot_cmd['values'], robot_cmd['unit'])
            if result == RobotJasonServerError.RESUTL_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error('modbus client write registers failed: host{0} address {1} value {2} unit {3}'.format(
                    robot_cmd['host'], robot_cmd['address'], robot_cmd['values'], robot_cmd['unit']))
        else:
            self.send_response(RobotJasonServerError.RESUTL_ERROR_ARGS, RobotResponse.failed)
            logger.error('modbus_client_write_registers bad args: args={0}'.format(robot_cmd))

    def _disconnect_to_modbus_server(self, robot_cmd):
        """
        * FUNCTION:    _disconnect_to_modbus_server
        * DESCRIPTION: 断开与modbus服务端的连接
        * INPUTS:      robot_cmd:机械臂控制指令
        * RETURNS:     成功返回: RobotJasonServerError.RESUTL_SUCC
        *              失败返回: 其它
        * EXAMPLE:     {"command":"disconnect_to_modbus_server","host":"192.168.1.30"}
        *              {"command":"disconnect_to_modbus_server","host":"/dev/ttyUSB0"}
        """
        if robot_cmd['host'] in self.robot_modbus_client:
            logger.info('disconnect to modbus server:{0}'.format(robot_cmd['host']))
            result = self.robot_modbus_client[robot_cmd["host"]].disconnect_to_modbus_server()
            self.robot_modbus_client.pop(robot_cmd["host"])
            if result == RobotJasonServerError.RESUTL_SUCC:
                self.send_response(RobotJasonServerError.RESUTL_SUCC, RobotResponse.succ)
            else:
                self.send_response(result, RobotResponse.failed)
                logger.error('disconnect to modbus server failed!')
        else:
            logger.error("Not connected to modbus server:{0}".format(robot_cmd['host']))
            self.send_response(RobotJasonServerError.RESUTL_ERROR_DISCONNECT_TO_MODBUS_SERVER, RobotResponse.failed)


def logger_init(logdir='./logfiles', logfile='./logfiles/json-server-python.log'):
    # Log等级总开关
    logger.setLevel(logging.INFO)

    # 创建log目录
    if not os.path.exists(logdir):
        os.mkdir(logdir)

    # 创建一个handler，用于写入日志文件
    # 以append模式打开日志文件
    fh = logging.FileHandler(logfile, mode='a')

    # 输出到file的log等级的开关
    fh.setLevel(logging.INFO)

    # 再创建一个handler，用于输出到控制台
    ch = logging.StreamHandler()

    # 输出到console的log等级的开关
    ch.setLevel(logging.INFO)

    # 定义handler的输出格式
    formatter = logging.Formatter("%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s: %(message)s")
    # formatter = logging.Formatter("%(asctime)s [%(thread)u] %(levelname)s: %(message)s")

    # 为文件输出设定格式
    fh.setFormatter(formatter)

    # 控制台输出设定格式
    ch.setFormatter(formatter)

    # 设置文件输出到logger
    logger.addHandler(fh)

    # 设置控制台输出到logger
    logger.addHandler(ch)


def run():
    try:
		#日志
        logger_init(logdir='./logfiles', logfile='./logfiles/json-server-python.log')
        
        #服务器
        robot_server = RobotJsonServer(9090, 'localhost', 8899)
        
        if robot_server.stratup() == RobotErrorType.RobotError_SUCC:
            
            while True:
				time.sleep(5)
     
            logger.info("shutdown")
            robot_server.shutdown()

    except RobotError, e:
        logger.error("robot start error:{0}".format(e))


if __name__ == '__main__':
    run()
