#! /usr/bin/env python
# coding=utf-8

import memcache
from pymodbus.interfaces import IModbusSlaveContext
from pymodbus.exceptions import ModbusException
from pymodbus.server.sync import StartTcpServer
from pymodbus.datastore import ModbusServerContext
# from pymodbus.utilities import pack_bitstring, unpack_bitstring
import threading
import logging
import json
import socket
_logger = logging.getLogger(__name__)


# Context
class MemcachedSlaveContext(IModbusSlaveContext):
    """
    This is a modbus slave context using memcached as a backing
    store.
    """

    def __init__(self, **kwargs):
        """ Initializes the datastores

        :param host: The host to connect to
        :param port: The port to connect to
        :param prefix: A prefix for the keys
        """
        self.prefix = kwargs.get('prefix', 'pymodbus')
        self.client = kwargs.get('client')
        self.__build_mapping()
        self.lock = threading.Lock()

    def __str__(self):
        """ Returns a string representation of the context

        :returns: A string representation of the context
        """
        return "Memcached Slave Context %s" % self.client

    def reset(self):
        """ Resets all the datastores to their default values """
        with self.lock:
            self.client.flushall()

    def validate(self, fx, address, count=1):
        """ Validates the request to make sure it is in range

        :param fx: The function we are working with
        :param address: The starting address
        :param count: The number of values to test
        :returns: True if the request in within range, False otherwise
        """
        _logger.debug("validate[%d] %d:%d" % (fx, address, count))
        with self.lock:
            return self.__val_callbacks[self.decode(fx)](address, count)

    def getValues(self, fx, address, count=1):
        """ Validates the request to make sure it is in range

        :param fx: The function we are working with
        :param address: The starting address
        :param count: The number of values to retrieve
        :returns: The requested values from a:a+c
        """
        _logger.debug("getValues[%d] %d:%d" % (fx, address, count))
        with self.lock:
            return self.__get_callbacks[self.decode(fx)](address, count)

    def setValues(self, fx, address, values):
        """ Sets the datastore with the supplied values

        :param fx: The function we are working with
        :param address: The starting address
        :param values: The new values to be set
        """
        _logger.debug("setValues[%d] %d:%d" % (fx, address, len(values)))
        with self.lock:
            self.__set_callbacks[self.decode(fx)](address, values)

    # Memcached Helper Methods
    def __get_prefix(self, key):
        """ This is a helper to abstract getting bit values

        :param key: The key prefix to use
        :returns: The key prefix to memcached
        """
        return "%s:%s" % (self.prefix, key)

    # 暂时只支持读取寄存器操作
    def __build_mapping(self):
        """
        A quick helper method to build the function
        code mapper.
        """
        self.__val_callbacks = {
            'd': lambda o, c: self.__val_bit('d', o, c),
            # 'c': lambda o, c: self.__val_bit('c', o, c),
            'c': lambda o, c: self.__val_reg('c', o, c),
            'h': lambda o, c: self.__val_reg('h', o, c),
            'i': lambda o, c: self.__val_reg('i', o, c),
        }
        self.__get_callbacks = {
            'd': lambda o, c: self.__get_bit('d', o, c),
            # 'c': lambda o, c: self.__get_bit('c', o, c),
            'c': lambda o, c: self.__get_reg('c', o, c),
            'h': lambda o, c: self.__get_reg('h', o, c),
            'i': lambda o, c: self.__get_reg('i', o, c),
        }
        self.__set_callbacks = {
            'd': lambda o, v: self.__set_bit('d', o, v),
            # 'c': lambda o, v: self.__set_bit('c', o, v),
            'c': lambda o, v: self.__set_reg('c', o, v),
            'h': lambda o, v: self.__set_reg('h', o, v),
            'i': lambda o, v: self.__set_reg('i', o, v),
        }

        # 用于解析memcached获取的存储信息，详细信息见文档
        self.__decode_handler = {}
        self.__decode_handler.update([(i, self.__get_robot_diagnose_info) for i in range(self.__address["diagnose_info_start"], self.__address["diagnose_info_end"] + 1)])
        self.__decode_handler.update([(i, self.__get_robot_joint_status) for i in range(self.__address["joint_status_start"], self.__address["joint_status_end"] + 1)])
        self.__decode_handler.update([(i, self.__get_robot_waypoint) for i in range(self.__address["waypoint_position_start"], self.__address["waypoint_ori_end"] + 1)])
        self.__decode_handler.update([(i, self.__get_robot_gravity_component) for i in range(self.__address["gravity_component_start"], self.__address["gravity_component_end"] + 1)])
        self.__decode_handler.update([(i, self.__get_robot_interface_board_do) for i in range(self.__address["interface_board_do_start"], self.__address["interface_board_do_end"] + 1)])
        self.__decode_handler.update([(i, self.__get_robot_interface_board_di) for i in range(self.__address["interface_board_di_start"], self.__address["interface_board_di_end"] + 1)])
        self.__decode_handler.update([(i, self.__get_robot_interface_board_ao) for i in range(self.__address["interface_board_ao_start"], self.__address["interface_board_ao_end"] + 1)])
        self.__decode_handler.update([(i, self.__get_robot_interface_board_ai) for i in range(self.__address["interface_board_ai_start"], self.__address["interface_board_ai_end"] + 1)])
        self.__decode_handler.update([(i, self.__get_virtual_holding_register) for i in range(self.__address["virtual_holding_register_start"], self.__address["virtual_holding_register_end"] + 1)])
        self.__decode_handler.update([(i, self.__get_virtual_DI) for i in range(self.__address["virtual_DI_start"], self.__address["virtual_DI_end"] + 1)])
        self.__decode_handler.update([(i, self.__get_virtual_DO) for i in range(self.__address["virtual_DO_start"], self.__address["virtual_DO_end"] + 1)])



    # Memcached discrete implementation
    __bit_size = 16
    __bit_default = '\x00' * (__bit_size % 8)

    def __get_bit_values(self, key, offset, count):
        """ This is a helper to abstract getting bit values

        :param key: The key prefix to use
        :param offset: The address offset to start at
        :param count: The number of bits to read
        """
        request = ["{0}:{1}".format(key, v) for v in range(offset, offset + count)]
        response = self.client.get_multi(request)
        return response

    def __val_bit(self, key, offset, count):
        """ Validates that the given range is currently set in memcached.
        If any of the keys return None, then it is invalid.

        :param key: The key prefix to use
        :param offset: The address offset to start at
        :param count: The number of bits to read
        """
        response = self.__get_bit_values(key, offset, count)
        return None not in response

    def __get_bit(self, key, offset, count):
        """

        :param key: The key prefix to use
        :param offset: The address offset to start at
        :param count: The number of bits to read
        """
        response = self.__get_bit_values(key, offset, count)
        '''
        response = (r or self.__bit_default for r in response)
        result = ''.join(response)
        result = unpack_bitstring(result)
        '''
        ret = [response[key] for key in response]
        return ret

    def __set_bit(self, key, offset, values):
        """

        :param key: The key prefix to use
        :param offset: The address offset to start at
        :param values: The values to set
        """
        count = len(values)
        """
        value = pack_bitstring(values)

        current = self.__get_bit_values(key, offset, count)
        current = (r or self.__bit_default for r in current)
        current = ''.join(current)
        current = current[0:offset] + value + current[offset + count:]
        final   = (current[s:s + self.__bit_size] for s in range(0, count, self.__bit_size))

        # key = self.__get_prefix(key)
        # request = ('%s:%s' % (key, v) for v in range(s, e + 1))
        # request = ('%s:%s' % (key, v) for v in range(offset, offset + count))
        # request = dict(zip(request, final))
        """
        # key = self.__get_prefix(key)
        headers = ["{0}:{1}".format(key, v) for v in range(offset, offset + count)]
        request = dict(zip(headers, values))
        self.client.set_multi(request)

    # Modbus数据地址映射
    __address = {
        "diagnose_info_start": 0,
        "diagnose_info_end": 27,
        "joint_status_start": 128,
        "joint_status_end": 169,
        "waypoint_position_start": 384,
        "waypoint_position_end": 386,
        "waypoint_ori_start": 387,
        "waypoint_ori_end": 390,
        "gravity_component_start": 448,
        "gravity_component_end": 450,
        "interface_board_do_start": 512,
        "interface_board_do_end": 559,
        "interface_board_di_start": 640,
        "interface_board_di_end": 691,
        "interface_board_ao_start": 768,
        "interface_board_ao_end": 771,
        "interface_board_ai_start": 896,
        "interface_board_ai_end": 899,
        "virtual_holding_register_start": 1000,
        "virtual_holding_register_end": 1127,
        "virtual_DI_start": 1256,
        "virtual_DI_end": 1383,
        "virtual_DO_start": 1512,
        "virtual_DO_end": 1639,
    }

    # Memcached key值信息对照表
    __robot_diagnose_info = ("canbus_status", "power_current", "power_voltage", "power_status", "controller_temp", "controller_humidity", "remote_halt", "soft_emergency", "remote_emergency", "robot_collision", "force_control_mode", "brake_stuats", "end_speed", "orpe_status", "enable_readpose", "mounting_pose_changed", "encoder_error_status", "static_collision_detect", "joint_collision_detect", "encoder_lines_error", "joint_error_status", "singularity_overSpeed_alarm", "current_alarm", "tool_io_err", "Mmounting_pose_warning", "mac_target_pos_buffer_size", "mac_target_pos_data_size", "mac_data_interrupt_warning")
    __jonit_status = ("joint_current", "joint_pos", "joint_cur_vol", "joint_cur_temp", "joint_tag_current", "joint_tag_pos", "joint_err_num")
    __position = ("x", "y", "z")
    __ori = ("w", "x", "y", "z")
    __gravity_component = ("x", "y", "z")

    def __get_reg_values(self, key, offset, count):
        """ This is a helper to abstract getting register values

        :param key: The key prefix to use
        :param offset: The address offset to start at
        :param count: The number of bits to read
        """
        request = ["{0}:{1}".format(key, v) for v in range(offset, offset + count)]
        response = self.client.get_multi(request)
        return response

    def __val_reg(self, key, offset, count):
        """ Validates that the given range is currently set in memcached.
        If any of the keys return None, then it is invalid.

        :param key: The key prefix to use
        :param offset: The address offset to start at
        :param count: The number of bits to read
        """
        # response = self.__get_reg_values(key, offset, count)
        response = self.__get_reg(key, offset, count)
        return None not in response

    def __set_reg(self, key, offset, values):
        """
        :param key: The key prefix to use
        :param offset: The address offset to start at
        :param values: The values to set
        """
        if 1000 <= offset <= 1127:
            key = "virtual_holding_register"
        elif 1256 <= offset <= 1383:
            key = "virtual_DI"
        elif 1512 <= offset <= 1639:
            key = "virtual_DO"

        count = len(values)
        offset -= self.__address[key + "_start"]
        headers = [key + "_{}".format(v) for v in range(offset, offset + count)]
        request = dict(zip(headers, values))
        self.client.set_multi(request)

    def __get_reg(self, key, offset, count):
        """
        :param key: The key prefix to use
        :param offset: The address offset to start at
        :param count: The number of bits to read
        """
        key = key
        response = [self.__get_single_reg_value(offset + i) for i in range(count)]
        return response

    # 获取单个寄存器值，并对地址有效性进行检验
    # 注意：所以数值均为无符号整形，如果想要转换到带符号整形值，请编程转换"if (val > 32768): val = val - 65535".
    def __get_single_reg_value(self, offset):
        """
        :param offset: The address offset to start at
        """
        if offset in self.__decode_handler:
            return self.__decode_handler[offset](offset)
        else:
            _logger.error("address is invalid!")
            raise ModbusException("address is invalid!")

    # 获取诊断信息
    def __get_robot_diagnose_info(self, offset):
        """
        :param offset: The address offset to start at
        """
        json_str = self.client.get_multi(["robot_diagnose_info"])
        robot_diagnose_info = json.loads(json_str["robot_diagnose_info"])
        result = robot_diagnose_info[self.__robot_diagnose_info[offset]]
        # 单位变换： 电流单位：mA  电压单位：mV   末端速度单位：mm/s
        if self.__robot_diagnose_info[offset] in ("power_current", "power_voltage", "end_speed"):
            result = result * 1000
        if result < 0:
            result = result + 65535
        return int(result)

    # 获取关节状态信息
    def __get_robot_joint_status(self, offset):
        """
        :param offset: The address offset to start at
        """
        json_str = self.client.get_multi(["robot_joint_status"])
        robot_joint_status = json.loads(json_str["robot_joint_status"])
        joint_id = (offset - self.__address["joint_status_start"]) // len(self.__jonit_status)
        joint_status = (offset - self.__address["joint_status_start"]) % len(self.__jonit_status)
        result = robot_joint_status["jonit_status"][joint_id][self.__jonit_status[joint_status]]
        # 关节角单位：mrad
        if self.__jonit_status[joint_status] in ("joint_pos", "joint_tag_pos"):
            result = result * 1000
        if result < 0:
            result = result + 65535
        return int(result)

    # 获取路点信息
    def __get_robot_waypoint(self, offset):
        """
        :param offset: The address offset to start at
        """
        json_str = self.client.get_multi(["robot_waypoint"])
        robot_waypoint = json.loads(json_str["robot_waypoint"])
        if self.__address["waypoint_position_start"] <= offset <= self.__address["waypoint_position_end"]:
            # 单位：um
            waypoint = robot_waypoint["position"][self.__position[offset - self.__address["waypoint_position_start"]]] * 10000
        else:
            # 单位：四元数原值的万分之一
            waypoint = robot_waypoint["ori"][self.__ori[offset - self.__address["waypoint_ori_start"]]] * 10000
        if waypoint < 0:
            waypoint = waypoint + 65535
        return int(waypoint)

    # 重力分量
    def __get_robot_gravity_component(self, offset):
        """
        :param offset: The address offset to start at
        """
        json_str = self.client.get_multi(["robot_gravity_component"])
        robot_gravity_component = json.loads(json_str["robot_gravity_component"])
        result = robot_gravity_component[self.__gravity_component[offset - self.__address["gravity_component_start"]]]
        # 重力分量单位：mm/s2
        result = result * 1000
        if result < 0:
            result = result + 65535
        return int(result)

    # 数字量输出
    def __get_robot_interface_board_do(self, offset):
        """
        :param offset: The address offset to start at
        """
        json_str = self.client.get_multi(["robot_interface_board_do"])
        robot_interface_board_do = json.loads(json_str["robot_interface_board_do"])
        return robot_interface_board_do["interface_board_do"][offset - self.__address["interface_board_do_start"]]["value"]

    # 数字量输入
    def __get_robot_interface_board_di(self, offset):
        """
        :param offset: The address offset to start at
        """
        json_str = self.client.get_multi(["robot_interface_board_di"])
        robot_interface_board_di = json.loads(json_str["robot_interface_board_di"])
        return robot_interface_board_di["interface_board_di"][offset - self.__address["interface_board_di_start"]]["value"]

    # 模拟量输出
    # 原始数据：前两位 0 - 10V      后两位 0 - 20mA
    # 转换后：  前两位 0 - 10000mV  后两位 0 - 20000uA
    def __get_robot_interface_board_ao(self, offset):
        """
        :param offset: The address offset to start at
        """
        json_str = self.client.get_multi(["robot_interface_board_ao"])
        robot_interface_board_ao = json.loads(json_str["robot_interface_board_ao"])
        result = robot_interface_board_ao["interface_board_ao"][offset - self.__address["interface_board_ao_start"]]["value"]
        return int(result * 1000)

    # 模拟量输入
    # 原始数据：0-10V
    # 转换后：0-10000mV
    def __get_robot_interface_board_ai(self, offset):
        """
        :param offset: The address offset to start at
        """
        json_str = self.client.get_multi(["robot_interface_board_ai"])
        robot_interface_board_ai = json.loads(json_str["robot_interface_board_ai"])
        result = robot_interface_board_ai["interface_board_ai"][offset - self.__address["interface_board_ai_start"]]["value"]
        # 模拟输入单位：mV
        return int(result * 1000)

    # 获取虚拟保持寄存器状态
    def __get_virtual_holding_register(self, offset):
        """
        :param offset: The address offset to start at
        """
        key = "virtual_holding_register_{}".format(offset - self.__address["virtual_holding_register_start"])
        result = self.client.get_multi([key])[key]
        return result

    # 获取虚DI状态
    def __get_virtual_DI(self, offset):
        """
        :param offset: The address offset to start at
        """
        key = "virtual_DI_{}".format(offset - self.__address["virtual_DI_start"])
        result = self.client.get_multi([key])[key]
        return result

    # 获取虚拟DO状态
    def __get_virtual_DO(self, offset):
        """
        :param offset: The address offset to start at
        """
        key = "virtual_DO_{}".format(offset - self.__address["virtual_DO_start"])
        result = self.client.get_multi([key])[key]
        return result

    # 工具端动力学参数:待增加：真实机械臂是否存在 是否是仿真模式 当前机械臂运动状态


def run():
    try:
        # 显示debug信息
        logging.basicConfig(level=logging.INFO)

        # 链接memcached服务端
        host = '127.0.0.1'
        port = 11211
        server = [str(host) + ':' + str(port)]
        memcache_client = memcache.Client(server)

        # 保证Memcached连接成功，否则弹出异常
        # if self.client.get_multi(["robot_waypoint"]) == {}:
        #     raise socket.error("Connected Memcached failed, ip:{0},port:{1}".format(host, port))
        if not memcache_client.get_stats():
            raise socket.error("Connected Memcached failed, ip:{0},port:{1}".format(host, port))

        # virtual_holding_register置零
        for i in range(0, 128):
            key = "virtual_holding_register_{}".format(i)
            memcache_client.set(key, 0)
            result = memcache_client.get_multi([key])[key]

        # virtual_DI置零
        for i in range(0, 128):
            key = "virtual_DI_{}".format(i)
            memcache_client.set(key, 0)
            result = memcache_client.get_multi([key])[key]

        # virtual_DO置零
        for i in range(0, 128):
            key = "virtual_DO_{}".format(i)
            memcache_client.set(key, 0)
            result = memcache_client.get_multi([key])[key]

        # 连接Memcached,创建存储上下文
        store = MemcachedSlaveContext(client=memcache_client)
        _logger.info("Connect to Memcached server succeeded.")

        # 创建server上下文
        context = ModbusServerContext(slaves=store, single=True)

        # 开启TCP Modbus Server 默认port:502
        _logger.info("Start TCP Modbus Server, ip: localhost port: 502")
        # StartTcpServer(context, address=("localhost", 502))
        StartTcpServer(context, address=("0.0.0.0", 502))

    except socket.error as e:
        _logger.error(e)
    except Exception as e:
        _logger.error("Exception:{0}".format(e))


if __name__ == '__main__':
    run()
